/**
 * Created by syedhusain on 10/30/13.
 */
function AdvanceDeviceManagement(basepath,temppath){
    /** @type {AppspaceLogger} */
    this.logger = null;
    this.appspaceURL = '';
    this.playerID = '';
    this.deviceType = '';
    this.basepath = '';
    this.tempPath = '';

    this.mainTimer = -1;

    this.deployedTasksCachePath = this.basepath + '/deployedtask.cache';
    this.deployedTaskExecutedHistoryPath = this.basepath + '/deployedtaskexecuted.cache';
    this.taskQueueTimeCachePath = this.basepath + '/taskqueuetime';

	
    /**@type {DeployedTask[]}*/
    this.deployedTasks = [];
    /**@type {TaskHistory[]}*/
    this.deployedTaskExecutionHistories = [];

    this.lastTaskQueueTime = 0;

    this.oriExecuteLoopSecondBlock =  20;
    this.executeRetrieveTaskQueuesSecondBlock = 2;
	this.executeTasksSecondBlock = 20;
	this.lastRetrieveTaskQueuesFromAppspaceTime = new Date();
	this.lastCheckExtensionAvailabilityTime = null;
	this.startDayTime = new Date();
    this.isRetrievingTask = false;
	this.pendingExecute = false;
    this.tasksExecuting = [];

    this.requireReboot = false;

    this.isExtensionAvailable = false;
    this.admExtensionUid = 'a340e2fd-0f7c-438b-931c-ed411d3bc078';
    this.isCheckingExtensionAvailability = false;
	
	
	this.environmentVars = [];
	this.environmentVars['ENV_APPSPACE_URL'] = '';
	this.environmentVars['ENV_PLAYER_GUID'] = '';
	this.environmentVars['ENV_TASK_ID'] = '';
	this.environmentVars['ENV_DEVICE_PASSWORD'] = '';

    this.lastFileWriteTime = null;
    this.fileWriteTimeTimeout = 2 * 60 * 1000;
}

AdvanceDeviceManagement.prototype.setLogger = function(logger){
    this.logger = logger;
}

AdvanceDeviceManagement.prototype.start = function(appspaceURL,playerID,deviceType){
	this.startDayTime = new Date();
	this.startDayTime.setHours(0);
	this.startDayTime.setMinutes(0);
	this.startDayTime.setSeconds(0);
	this.startDayTime.setMilliseconds(0);
    if (appspaceURL.toLowerCase().indexOf('http://') == -1 && appspaceURL.toLowerCase().indexOf('https://') == -1){
		appspaceURL = 'http://' + appspaceURL;
	}
	
    if (appspaceURL.charAt(appspaceURL.length - 1) == '/')
        appspaceURL = appspaceURL.substring(0,appspaceURL.length - 1);

    this.logger.log('info','[ADM] start ('+appspaceURL+','+playerID+','+deviceType+')');

    this.appspaceURL = appspaceURL;
    this.playerID = playerID;
    this.deviceType = this.resolveDeviceType(deviceType);

	switch(this.deviceType)
	{
		case 'edge340' : this.environmentVars['ENV_DEVICE_PASSWORD'] = 'aDMIN123#'; break;
		case 'edge300' : this.environmentVars['ENV_DEVICE_PASSWORD'] = 'cisco'; break;
		case 'dmp4310' : this.environmentVars['ENV_DEVICE_PASSWORD'] = 'Cisconexus1'; break;
		case 'dmp4400' : this.environmentVars['ENV_DEVICE_PASSWORD'] = 'Cisconexus1'; break;
	}

    if (this.deviceType == 'edge340' || this.deviceType == 'edge300'){
        this.basepath = '/apps/nexus2';
        this.tempPath = '/tmp/appspace/appspaceadm';
    } else if (this.deviceType == 'dmp4310' || this.deviceType == 'dmp4400'){
        this.basepath = '/tmp/ftproot/usb_1'
        this.tempPath = '/tmp/ftproot/usb_1/appspaceadm';
    }

	
    /*console.log('base path:' + this.basepath);
    console.log('temp path:' + this.tempPath);
    console.log('device type:' + this.deviceType);*/


    this.deployedTasksCachePath = this.tempPath + '/deployedtask.cache';
    this.deployedTaskExecutedHistoryPath = this.tempPath + '/deployedtaskexecuted.cache';
    this.taskQueueTimeCachePath = this.tempPath + '/taskqueuetime';

    this.deployedTasksCachePermanentPath = this.basepath + '/deployedtask.cache';
    this.deployedTaskExecutedHistoryPermanentPath = this.basepath + '/deployedtaskexecuted.cache';
    this.taskQueueTimeCachePermanentPath = this.basepath + '/taskqueuetime';
	
	this.environmentVars['ENV_PLAYER_GUID'] = playerID;
	this.environmentVars['ENV_APPSPACE_URL'] = appspaceURL;


	
    this.transferFilesFromPermanentToCache(EventHandler.CreateEventHandler(this,this.loadProxyConfigurationHandler));
    //this.loadTaskQueueTimeFromCache();
	//setTimeout(EventHandler.CreateEventHandler(this,this.loadTaskQueueTimeFromCacheHandler),5000);
}

AdvanceDeviceManagement.prototype.transferFilesFromPermanentToCache = function(callback){
    this.logger.log('info','[ADM] transfer ADM files from ' + this.basepath + ' to ' + this.tempPath);
    fsextra.mkdirRecursive(this.tempPath,function(err){});
	
	var addresses = {};
	addresses[this.deployedTasksCachePermanentPath] = this.deployedTasksCachePath;
	addresses[this.deployedTaskExecutedHistoryPermanentPath] = this.deployedTaskExecutedHistoryPath;
	addresses[this.taskQueueTimeCachePermanentPath] = this.taskQueueTimeCachePath;
	
	this.transFunc(addresses,callback);
}

AdvanceDeviceManagement.prototype.transFunc = function(addresses, callback){
	var owner = this;
	var src = target = null;
	for (var addr in addresses){
		if(addresses[addr] != null){
			src = addr;
			target = addresses[addr];
			addresses[addr] = null;
			delete addresses[addr];
			break;
		}
	}
	if(src != null && target != null){
		var stream = fs.createReadStream(src);
		stream.on('error',function(err){
			owner.logger.log('error','[ADM] error on copying file ' + src + ' to ' + target);
			owner.logger.log('error',err);
			owner.transFunc(addresses,callback);
		});
		stream.on('close',function(){
			owner.transFunc(addresses, callback);
		});
		stream.pipe(fs.createWriteStream(target));
	}else{
		if(callback!= null){
			callback();
		}
	}
}

AdvanceDeviceManagement.prototype.transferFilesFromCacheToPermanent = function(){
    this.logger.log('info','[ADM] transfer ADM files from ' + this.tempPath + ' to ' + this.basepath);
    this.lastFileWriteTime = new Date();

	var addresses = {};
	addresses[this.deployedTasksCachePath] = this.deployedTasksCachePermanentPath;
	addresses[this.deployedTaskExecutedHistoryPath] = this.deployedTaskExecutedHistoryPermanentPath;
	addresses[this.taskQueueTimeCachePath] = this.taskQueueTimeCachePermanentPath;
	
	this.transFunc(addresses,null);
}

AdvanceDeviceManagement.prototype.loadProxyConfigurationHandler = function(calle){
	calle.loadProxyConfiguration();
}

AdvanceDeviceManagement.prototype.loadProxyConfiguration = function(){
	//check for proxy here
	var proxy = this.retrieveProxy(EventHandler.CreateEventHandler(this,this.onProxyRetrievedHandler));
}

AdvanceDeviceManagement.prototype.onProxyRetrievedHandler = function(calle,proxy){
	calle.onProxyRetrieved(proxy);
}

AdvanceDeviceManagement.prototype.onProxyRetrieved = function(proxy){
	this.useProxy(proxy);
	this.loadTaskQueueTimeFromCache();
}

AdvanceDeviceManagement.prototype.useProxy = function (proxy){
	//proxy
	/*var proxy = {
		host: "",
		user: "",
		password: "",
		exception: ""
	}*/
	
	if (proxy == null){
		this.logger.log('info','[ADM] No proxy settings detected');
		return;
	}
	
	if(proxy.host == null || proxy.host == ""){
		return;
	}
	
	if (proxy.exception != undefined && proxy.exception != ""){
		this.logger.log('info','[ADM] Setting proxy exception:' + proxy.exception);
		process.env.NO_PROXY = proxy.exception;
		process.env.no_proxy = proxy.exception;
	}
	
	
	var userPass = "";
	if(proxy.user != null && proxy.user != ""){
		userPass = proxy.user + ":" + proxy.password + "@";
	}
	
	var suffix = (proxy.host.toLowerCase().indexOf("https://") > -1) ? "https://" : "http://";
	var host = proxy.host.toLowerCase().split("https://").join("").split("http://").join("");
	var proxyURL = suffix+userPass+host;
	this.logger.log('info','[ADM] Setting proxy:' + proxyURL);
	console.log(proxyURL);
	RequestUtil.setProxy(proxyURL);
}

AdvanceDeviceManagement.prototype.retrieveProxy = function(callback){
	this.logger.log('info','[ADM] retrieving proxy for ' + this.deviceType);	
	switch(this.deviceType){
		case 'edge340' : this.retrieveE340Proxy(callback); break;
		case 'edge300' : this.retrieveE300Proxy(callback); break;
		case 'dmp4310' :
		case 'dmp4400' : this.retrieveDMPProxy(callback); break;
	}
}

AdvanceDeviceManagement.prototype.retrieveE340Proxy = function(callback){
	var checkurl = "https://127.0.0.1/api/v3/system/proxy";
	this.logger.log('info','[ADM] retrieving proxy info : ' + checkurl);
	var owner = this;
    var options = RequestUtil.createOptionsObject();
    options.url = checkurl;
    options.failCallback = owner.retrieveE340ProxyFail;
    options.successCallback = owner.retrieveE340ProxySuccess;
	options.state = callback;	

	RequestUtil.makeRequest(options);
}

AdvanceDeviceManagement.prototype.retrieveE340ProxySuccess = function(res,body,callback){
	var proxyObject = JSON.parse(body);
	var proxy = null;
	if (proxyObject.http && proxyObject.https && proxyObject.bypass){
		proxy = {};
		if (proxyObject.http != ''){
			proxy.host = 'http://' + proxyObject.http;
		}else if (proxyObject.https != ''){
			proxy.host = 'https://' + proxyObject.https;
		}
		
		if (proxyObject.bypass.length > 0){
			proxy.exception = proxyObject.bypass.join(',');
		}
	}
	
	console.log(proxy);
	if (callback != null)
		callback(proxy);
}

AdvanceDeviceManagement.prototype.retrieveE340ProxyFail = function(err,callback){
	if (callback != null)
		callback(null);
}

AdvanceDeviceManagement.prototype.retrieveE300Proxy = function(callback){	
	//in edge 300, Setting a proxy through api will set the environment variable which request library respect. so we don't need to do anything.	
	callback(null);
}
AdvanceDeviceManagement.prototype.retrieveDMPProxySuccess = function(res, body, callback){
	var lines = body.split('\r\n');
	var proxy = null;
	var proxyon = false;
	var proxyip = "";
	var proxyport = 0;
	var proxyexception = "";
	for (var i = 0 ; i < lines.length; i++){
		var datas = lines[i].split('\t');				
		if (datas.length == 3){
			switch(datas[0].toLowerCase()){
				case "init.browser_proxy":
					proxyon = (datas[2].toLowerCase() == "off")?false:true;
					break;
				case "init.browser_proxy_ip":
					proxyip = datas[2];
					break;
				case "init.browser_proxy_port":
					proxyport = datas[2] * 1;
					break;
				case "init.browser_no_proxy":
					proxyexception = datas[2];
					break;
			}
		}
	}
	
	if (proxyon == true){
		proxy = { 
			host: proxyip + ":" + proxyport,
			exception: proxyexception
		};
	}
	if(callback != null)
		callback(proxy);
}
AdvanceDeviceManagement.prototype.retrieveDMPProxyFail = function(err, callback){
	var proxy = null;
	if(callback != null){
		callback(proxy);
	}
}

AdvanceDeviceManagement.prototype.retrieveDMPProxy = function(callback){
	
	var checkurl = "http://127.0.0.1:7777/get_param?p=init.BROWSER_PROXY&p=init.BROWSER_PROXY_IP&p=init.BROWSER_PROXY_PORT&p=init.BROWSER_NO_PROXY";
	this.logger.log('info','[ADM] retrieving proxy info : ' + checkurl);

	var owner = this;
    var options = RequestUtil.createOptionsObject();
    options.url = checkurl;
    options.failCallback = owner.retrieveDMPProxyFail;
    options.successCallback = owner.retrieveDMPProxySuccess;
	options.state = callback;	

	RequestUtil.makeRequest(options);
}

AdvanceDeviceManagement.prototype.loadTaskQueueTimeFromCacheHandler = function(calle){
	calle.loadTaskQueueTimeFromCache();
}

AdvanceDeviceManagement.prototype.loadTaskQueueTimeFromCache = function(){
    this.logger.log('info','[ADM] loadTaskQueueTimeFromCache');
    fs.readFile(this.taskQueueTimeCachePath,EventHandler.CreateEventHandler(this,this.onReceiveTaskQueueTimeFromCacheHandler));
}

/**
 *
 * @param {AdvanceDeviceManagement} calle
 * @param {Error} err
 * @param {String} data
 */
AdvanceDeviceManagement.prototype.onReceiveTaskQueueTimeFromCacheHandler = function(calle,err,data){
    calle.onReceiveTaskQueueTimeFromCache(err,data);
}

/**
 *
 * @param {Error} error
 * @param {String} res
 */
AdvanceDeviceManagement.prototype.onReceiveTaskQueueTimeFromCache = function(error,res){

    if (error != null && error != undefined){
        this.lastTaskQueueTime = 0;
    }else{
		res = res.toString();
		
		if (res == null || res == "" || res == "null" || isNaN(parseInt(res,10)) == true)
			res = "0";
		
        this.lastTaskQueueTime = parseInt(res,10);
		
		if (isNaN(this.lastTaskQueueTime) == true)
			this.lastTaskQueueTime = 0;
    }
    this.logger.log('info','[ADM] last task queue time : ' + this.lastTaskQueueTime);
    this.loadDeployedTasksFromCache();
}

AdvanceDeviceManagement.prototype.loadDeployedTasksFromCache = function(){
    this.logger.log('info','[ADM] loadDeployedTasksFromCache');
    fs.readFile(this.deployedTasksCachePath,EventHandler.CreateEventHandler(this,this.onReceiveDeployedTasksFromCacheHandler));
}

/**
 *
 * @param {AdvanceDeviceManagement} calle
 * @param {Error} error
 * @param {String} data
 */
AdvanceDeviceManagement.prototype.onReceiveDeployedTasksFromCacheHandler = function(calle,error,data){
    calle.onReceiveDeployedTasksFromCache(error,data);
}

AdvanceDeviceManagement.prototype.onReceiveDeployedTasksFromCache = function(error,data){
    if (error == undefined && error == null && data.toString() != ''){
        this.deployedTasks = JSON.parse(data);
    }
    this.logger.log('info','[ADM] # deployedTasks loaded from cache : ' + this.deployedTasks.length);
    this.loadTaskExecutionHistoryFromCache();
}

AdvanceDeviceManagement.prototype.loadTaskExecutionHistoryFromCache = function(){
    this.logger.log('info','[ADM] loadTaskExecutionHistoryFromCache');
    fs.readFile(this.deployedTaskExecutedHistoryPath,EventHandler.CreateEventHandler(this,this.onReceiveTaskExecutionHistoryHandler));
}

/**
 *
 * @param {AdvanceDeviceManagement} calle
 * @param {Error} error
 * @param {String} data
 */
AdvanceDeviceManagement.prototype.onReceiveTaskExecutionHistoryHandler = function(calle,error,data){
    calle.onReceiveTaskExecutionHistory(error,data);
}

AdvanceDeviceManagement.prototype.onReceiveTaskExecutionHistory = function(error,data){

    if ((error == undefined || error == null) && data.toString() != ''){
        this.deployedTaskExecutionHistories = JSON.parse(data);
    }
    this.logger.log('info','[ADM] # task execution histories loaded from cache : ' + this.deployedTaskExecutionHistories.length);
    this.executeTaskOnBoot();
}

AdvanceDeviceManagement.prototype.executeTaskOnBoot = function(){
    this.logger.log('info','[ADM] executeTaskOnBoot');
    var l = this.deployedTasks.length;
    for (var i = 0 ; i < l ; i++){
        var task = this.deployedTasks[i];
        //TaskScheduleType:Undefined = 0,Run_Once = 1,Run_Time = 2,Run_Daily = 3,Run_Weekly = 4,Run_Monthly = 5,On_Idle = 6,On_SystemStart = 7,On_SystemStop = 8,On_Frequency = 9,On_Demand = 10,
        if (task.TaskScheduleType == 7){
            this.executeTask(task);
        }
    }
	this.loadPlayerProperties();
    //this.starttimer();

}

AdvanceDeviceManagement.prototype.loadPlayerProperties = function(){
	var file = this.basepath + "/debugScript.txt";
	var callback = EventHandler.CreateEventHandler(this,this.onPlayerScriptLoadedHandler);
	fs.readFile(file,callback);
}

AdvanceDeviceManagement.prototype.onPlayerScriptLoadedHandler = function(calle,err,data){
	calle.onPlayerScriptLoaded(err,data);
}

AdvanceDeviceManagement.prototype.onPlayerScriptLoaded = function(err,data){
	this.logger.log('info','[ADM] retrieving ADM-specific properties from script');
	if (err != null){
		this.executeRetrieveTaskQueuesSecondBlock = this.oriExecuteLoopSecondBlock;
		this.starttimer();
		return;
	}
	
	data = data.toString();
	//get player properties section
	var startIndex = data.toLowerCase().indexOf("<id>applicationproperties");
	var endIndex = data.toLowerCase().indexOf("</id>",startIndex);
	startIndex = endIndex + 5;
	endIndex = data.toLowerCase().indexOf("</table>",startIndex);
	data = data.substring(startIndex,endIndex);
	var props = data.split("\r\n").join("").split(";");
	var l = props.length;
	var intervalDuration = 20;
	for (var i = 0 ; i < l; i++){
		var raw = props[i];
		if (raw == '')
			continue;
		var kvps = raw.split("=");
		var key = kvps[0].toLowerCase();
		var value = kvps[1];
		switch(key){
			case "player.adm.tasks.polling.rate"://in seconds
				intervalDuration = parseInt(value,10);
				break;
		}
	}
	
	if (intervalDuration < 10)
		intervalDuration = 10;
	if (intervalDuration >= 86400)
		intervalDuration = 0;
	
	this.logger.log('info','[ADM] Retrieve Task Queue loop rate (seconds):' + intervalDuration);
	this.executeRetrieveTaskQueuesSecondBlock = intervalDuration;	
	this.starttimer();
}

AdvanceDeviceManagement.prototype.starttimer = function(){
    this.mainTimer = setTimeout(EventHandler.CreateEventHandler(this,this.onTimerTickHandler),1000);
}

AdvanceDeviceManagement.prototype.onTimerTickHandler = function(calle){
	clearTimeout(calle.mainTimer);
    calle.onTimerTick();
	calle.starttimer();
}

AdvanceDeviceManagement.prototype.onTimerTick = function(){
	
	try
	{
		var c = new Date();
		

        if (this.lastFileWriteTime == null || (this.lastFileWriteTime != null && (c.getTime() - this.lastFileWriteTime.getTime()) >= this.fileWriteTimeTimeout)){
            this.transferFilesFromCacheToPermanent();
        }
			
		if (this.isExtensionAvailable == false){//if extension is not available, don't execute anything else
			if (this.lastCheckExtensionAvailabilityTime == null || (c.getTime() - this.lastCheckExtensionAvailabilityTime.getTime()) > (60*10*1000)){//if already past 10 minutes since last check, check extension availability again
				if (this.isCheckingExtensionAvailability == false) this.checkIfExtensionAvailable();
			}
			return;
		}
		
		if (this.startDayTime.getDate() != c.getDate() || this.startDayTime.getMonth() != c.getMonth() || this.startDayTime.getFullYear() != c.getFullYear()){
			this.startDayTime = new Date(c.getFullYear(),c.getMonth(),c.getDate(),00,00,00,00);
		}
		
		var secondElapsedSinceDayStart = parseInt((c.getTime() - this.startDayTime.getTime())/1000,10);
		//console.log(secondElapsedSinceDayStart);
		//console.log(this.executeRetrieveTaskQueuesSecondBlock);
		
		if (this.isRetrievingTask == false && (secondElapsedSinceDayStart % this.executeRetrieveTaskQueuesSecondBlock == 0)){
			this.retrieveTaskQueueFromAppspace();
		}
		
		var executeTask = false;
		
		if (secondElapsedSinceDayStart % this.executeTasksSecondBlock == 0){//time to execute tasks
			if (this.isRetrievingTask == true){//we're still retrieving task from Appspace,delay execute to avoid task conflictions
				this.logger.log('info','[ADM] Delay tasks executions, still retrieving task from Appspace');
				this.pendingExecute = true;
				executeTask = false;
			}else{
				executeTask = true;
			}
		}
		
		if (this.isRetrievingTask == false && this.pendingExecute == true) executeTask = true; //we're not retrieving task but we have delay execute flag, so we need to execute
		
		if (executeTask == true) {
			this.pendingExecute = false;
			this.executeTasksIfValid();
		}
		
		/*if (this.isExtensionAvailable == false && pastElapsed == true){
			this.lastRetrieveTaskQueuesFromAppspaceTime = new Date();
			if (this.isCheckingExtensionAvailability == false){
				this.checkIfExtensionAvailable();
			}
		}
		else if (this.requireReboot == false && this.isRetrievingTask == false && pastElapsed == true){
			this.lastRetrieveTaskQueuesFromAppspaceTime = new Date();
			this.retrieveTaskQueueFromAppspace();
		}
		else if (this.requireReboot == true){
			var rebootnow = true;

			for ( var key in this.tasksExecuting){
				if (this.tasksExecuting[key] != null){
					rebootnow = false;
					break;
				}
			}

			if (rebootnow == true){
				this.logger.log('info','rebooting device');
                this.transferFilesFromCacheToPermanent();
				exec('reboot -f -n');
			}
		}*/
	}
	catch(e)
	{
		this.logger.log('error','Error on main loop');
		this.logger.log('error', e.message);
	}
	
   
}

AdvanceDeviceManagement.prototype.checkIfExtensionAvailable = function(){
    this.logger.log('info','[ADM] Checking if ADM available on Appspace');
    this.isCheckingExtensionAvailability = true;
	this.lastCheckExtensionAvailabilityTime = new Date();
    var url = this.appspaceURL;
    var param = '/api/v1/appspace/extensions/checkextensionexistance/' + this.admExtensionUid;


    var options = RequestUtil.createOptionsObject();
    options.url = url + param;
    options.failCallback = EventHandler.CreateEventHandler(this, this.errorCheckIfExtensionAvailable);
    options.successCallback = EventHandler.CreateEventHandler(this,this.onReceiveResponseIfExtensionAvailableHandler);

    this.logger.log('info','[ADM] Requesting ' + options.url);

	RequestUtil.makeRequest(options);
}

AdvanceDeviceManagement.prototype.errorCheckIfExtensionAvailable = function(calle, err){
	console.log(err);
	calle.onExtensionResponseFail('[ADM] Receive error : ' + err.Message);
}

AdvanceDeviceManagement.prototype.onReceiveResponseIfExtensionAvailableHandler = function(calle,req,body){
	calle.onReceiveResponseIfExtensionAvailable(req,body);
}

AdvanceDeviceManagement.prototype.onReceiveResponseIfExtensionAvailable = function(req,body){
    var owner = this;
    if((req.statusCode >= 200 && req.statusCode < 300) || req.statusCode === 304){
        var data = '';
       	data+= body.toString();
		owner.logger.log('info','[ADM] Receive ' + data);
		if (data.toString().toLowerCase().indexOf('success') > -1){
			//owner.executeRetrieveTaskQueuesSecondBlock = owner.oriExecuteLoopSecondBlock;
			owner.isExtensionAvailable = true;
		}
		else{
			//owner.executeRetrieveTaskQueuesSecondBlock = 60 * 10;
			owner.isExtensionAvailable = false;
		}
		//owner.logger.log('info','[ADM] Setting flag to ' + owner.isExtensionAvailable + ' and interval to ' + owner.executeRetrieveTaskQueuesSecondBlock);			
		owner.logger.log('info','[ADM] Setting extension availability flag to ' + owner.isExtensionAvailable);
		owner.isCheckingExtensionAvailability = false;
		//req.socket.end()
    }
    else{
		this.onExtensionResponseFail('Status Code ' + res.statusCode);
    }
}

AdvanceDeviceManagement.prototype.onExtensionResponseFail = function(errormessage){
	this.logger.log('info','[ADM] Receive error on checking extension : ' + errormessage);
	this.logger.log('info','[ADM] Re-checking in 10 minutes');
	//this.executeRetrieveTaskQueuesSecondBlock = 60 * 10;
	this.isExtensionAvailable = false;
	//this.logger.log('info','[ADM] Setting flag to ' + this.isExtensionAvailable + ' and interval to ' + this.executeRetrieveTaskQueuesSecondBlock);
	this.isCheckingExtensionAvailability = false;
}

AdvanceDeviceManagement.prototype.retrieveTaskQueueFromAppspace = function(){
    this.logger.log('info','[ADM] retrieveTaskQueueFromAppspace');
    this.isRetrievingTask = true;
    var url = this.appspaceURL;
	
    var param = '/api/v1/core/tasks/queue/' + this.playerID + '/' + this.lastTaskQueueTime;
    
	var owner = this;
    //----------------------------------------------------
	var Curl = this.appspaceURL + param;
	this.logger.log('info','[ADM] retrieve task queues from Appspace ' + Curl);
	
	var options = RequestUtil.createOptionsObject();
    options.url = Curl;
    options.headers = { 'Content-Type' : 'application/json' };
    options.method = 'GET';
    options.failCallback = EventHandler.CreateEventHandler(this,this.onReceiveTaskQueueFromAppspaceErrorHandler);
    options.successCallback = EventHandler.CreateEventHandler(this,this.onReceiveTaskQueueFromAppspaceHandler);

	RequestUtil.makeRequest(options);
}

AdvanceDeviceManagement.prototype.onReceiveTaskQueueFromAppspaceHandler = function(calle,res, body){
    var owner = calle;
	owner.logger.log('info','[ADM] receive status code : ' + res.statusCode);
    if((res.statusCode >= 200 && res.statusCode < 300) || res.statusCode === 304){
        var data = '';
		data+= body.toString();
		//res.ondata(function(){console.log("DATA !");});
        //res.on('data',function(chunk){data += chunk.toString();});
        owner.onReceiveTaskQueueFromAppspace(data);
		//console.log(data);
    }
	else{
		var err = { message : 'Receive ' + res.statusCode + ' code.' };
		owner.onReceiveTaskQueueFromAppspaceErrorHandler(calle,err);
	}
}

AdvanceDeviceManagement.prototype.onReceiveTaskQueueFromAppspaceErrorHandler = function(calle,err){
	calle.logger.log('info','[ADM] receive error : ' + err.message);
    calle.retrieveTaskQueueCompleted();
}

AdvanceDeviceManagement.prototype.onReceiveTaskQueueFromAppspace = function(data){
    this.logger.log('info','[ADM] Receive data with length ' + data.length + ' from Appspace');

    var l = 0;
    /** @type {TaskQueue[]} */
    var taskQueues = [];

    try{
        var jsonObject = JSON.parse(data.toString());
        taskQueues = jsonObject.TaskQueues;
        if (taskQueues != null && taskQueues != undefined)
            l = taskQueues.length;
    }
    catch (e){
        this.logger.log('error', e.message);
        taskQueues = [];
        l = 0;
    }

    this.logger.log('info','[ADM] Received ' + l + ' task queue(s)');


	
    var executedCount = 0;
	var maxSockets = l;
	if (maxSockets < 20)
		maxSockets = 20;
    http.globalAgent.maxSockets = maxSockets;
    https.globalAgent.maxSockets = maxSockets;
	
    for (var i = 0 ; i < l ; i++){
        var taskQueue = taskQueues[i];
        var executed = true;
		
		if (taskQueue == null)
			continue;
		
		if (taskQueue.TaskQueueCommand == null)
			taskQueue.TaskQueueCommand = '';
			
        switch(taskQueue.TaskQueueCommand.toLowerCase()){
            case 'add':
                this.logger.log('info','[ADM] (TQID:'+ taskQueue.Id +') Adding/Updating ' + taskQueue.DeployedTask.Name + ' (Id:' + taskQueue.DeployedTask.Id + ') to local cache...');
				var ll = this.deployedTasks.length;
				var found = false;
				for (var j = 0 ; j < ll ; j++){
					var dTask = this.deployedTasks[j];
					if (dTask.Id == taskQueue.DeployedTask.Id){
						this.deployedTasks[j] = taskQueue.DeployedTask;
						found = true;
						this.logger.log('info','[ADM] ' + taskQueue.DeployedTask.Name + ' (Id:' + taskQueue.DeployedTask.Id + ') updated');
					}
				}
				if (found == false){
					this.deployedTasks.push(taskQueue.DeployedTask);
					this.logger.log('info','[ADM] ' + taskQueue.DeployedTask.Name + ' (Id:' + taskQueue.DeployedTask.Id + ') added');
				}
                break;
            case 'remove':
                var ll = this.deployedTasks.length;
                var indexToRemove = -1;
                for (var j = 0 ; j < ll ; j++){
                    var dTask = this.deployedTasks[j];
                    if (dTask.Id == taskQueue.DeployedTask.Id){
                        indexToRemove = j;
                        break;
                    }
                }
                if (indexToRemove != -1){
                    this.logger.log('info','[ADM] (TQID:'+ taskQueue.Id +') Removing ' + taskQueue.DeployedTask.Name + ' (Id:' + taskQueue.DeployedTask.Id + ') from local cache')
                    this.deployedTasks.splice(j,1);
                }

                break;
            case 'execute':
                var ll = this.deployedTasks.length;
                for (var j = 0 ; j < ll ;j++){
                    var dTask = this.deployedTasks[j];
                    if (dTask.Id == taskQueue.DeployedTask.Id){
                        this.logger.log('info','[ADM] (TQID:'+ taskQueue.Id +') Executing task ' + taskQueue.DeployedTask.Name + ' (Id:' + taskQueue.DeployedTask.Id + ') in local cache');
                        this.executeTask(dTask);
                        break;
                    }
                }
                break;
            default:
                executed = false;
				this.logger.log('info','[ADM] Task queue (' + taskQueue.Id + ') not executed. Unknown command:' + taskQueue.TaskQueueCommand);
                break;
        }
        if (executed == true){
            executedCount++;
            var lastExecutedTime = Parser.parseServerDate(taskQueue.QueueDate);
            if (lastExecutedTime.getTime() > this.lastTaskQueueTime){
                this.lastTaskQueueTime = lastExecutedTime.getTime();
            }
			var owner = this;
			var url = this.appspaceURL;            
			var param = '/api/v1/core/tasks/queue/' + taskQueue.Id + '/' + this.playerID + '/ack';
            //----------------------------------------------------
			var Curl = this.appspaceURL + param;
			this.logger.log('info','[ADM] Send task queue acknowledgement ' + Curl);
			
			var options = RequestUtil.createOptionsObject();
		    options.url = Curl;
		    options.headers = { 'Content-Type' : 'application/json' };
		    options.method = 'POST';
		    options.failCallback = EventHandler.CreateEventHandler(this, this.errorOnReceiveTaskQueueFromAppspace);

			RequestUtil.makeRequest(options);
        }
    }
    this.logger.log('info','[ADM] Executed ' + executedCount + ' task queue');
	if (executedCount != 0)
		this.retrieveTaskQueueCompleted();
	else//no task queues retrieved
		this.isRetrievingTask = false;
}

AdvanceDeviceManagement.prototype.errorOnReceiveTaskQueueFromAppspace = function(error){
	console.log('error - ' + error.message);
}

/**
 *
 * @param {AdvanceDeviceManagement} calle
 * @param {TaskQueue} state
 * @param res
 */
AdvanceDeviceManagement.prototype.callbackForSendTQAckHandler = function(calle,state,res){
	res.socket.end();
    /*if (res.statusCode == 200)
        res.on('data',function(data){console.log(data.toString())});*/
}

AdvanceDeviceManagement.prototype.retrieveTaskQueueCompleted = function(){
    this.saveLastTaskQueueTime();
}

AdvanceDeviceManagement.prototype.saveLastTaskQueueTime = function(){
    this.logger.log('info','[ADM] Saving task queue time to file')
    fs.writeFile(this.taskQueueTimeCachePath,JSON.stringify(this.lastTaskQueueTime),EventHandler.CreateEventHandler(this,this.onLastQueueTimeSavedHandler));
}

AdvanceDeviceManagement.prototype.onLastQueueTimeSavedHandler = function(calle,error){
    calle.onLastQueueTimeSaved(error);
}

AdvanceDeviceManagement.prototype.onLastQueueTimeSaved = function(error){

    if (error == null || error == undefined){
        this.saveDeployedTasksToCache();
    }
    else{
        this.isRetrievingTask = false;
    }
}

AdvanceDeviceManagement.prototype.saveDeployedTasksToCache = function(){
    this.logger.log('info','[ADM] Saving deployed tasks to file')
    var output = JSON.stringify(this.deployedTasks);
    fs.writeFile(this.deployedTasksCachePath,output,EventHandler.CreateEventHandler(this,this.onDeployedTasksSavedHandler));
}

AdvanceDeviceManagement.prototype.onDeployedTasksSavedHandler = function(calle,error){
    calle.onDeployedTasksSaved(error);
}

AdvanceDeviceManagement.prototype.onDeployedTasksSaved = function(error){
	this.isRetrievingTask = false;
}

AdvanceDeviceManagement.prototype.executeTasksIfValid = function(){
    this.logger.log('info','[ADM] Looking for valid deployed tasks to be executed...')
    var l = this.deployedTasks.length;
    for (var i = 0 ; i < l ; i++){
        var task = this.deployedTasks[i];
        if (task.TaskScheduleType == 1 || task.TaskScheduleType == 2 || task.TaskScheduleType == 9){
            if (task.RequireReboot == true)
                this.requireReboot = true;
        }
        //TaskScheduleType:Undefined = 0,Run_Once = 1,Run_Time = 2,Run_Daily = 3,Run_Weekly = 4,Run_Monthly = 5,On_Idle = 6,On_SystemStart = 7,On_SystemStop = 8,On_Frequency = 9,On_Demand = 10,
        switch(task.TaskScheduleType){
            case 1:
				var execute = true;
				if (this.getLastTaskHistory(task.Id) != null){
					execute = false;
				} else if (task.Schedule.EndBoundary != null){
					var endBound = Parser.parseServerDate(task.Schedule.EndBoundary);
					var c = new Date();
					if (c.getTime() > endBound.getTime()){
						execute = false;
					}
				}
                
				if (execute == true)
					this.executeTask(task);
					
                break;
            case 2:
                this.executeTaskOnTime(task);
                break;
			case 3:
			case 4:
			case 5:
				this.executeTaskOnDailyWeeklyMonthly(task);
				break;
            case 9:
                this.executeTaskOnFrequency(task);
                break;
            default:
				this.logger.log('warning','[ADM] Task ' + task.Name + ' (Id:' + task.Id + ') not executed. TaskScheduleType Undefined');
                break;
        }
    }
    this.isRetrievingTask = false;
}
/**
 *
 * @param {DeployedTask} task
 */
AdvanceDeviceManagement.prototype.executeTaskOnTime = function(task){
    var lastExecute = this.getLastTaskHistory(task.Id);
    var lastExecuteDate = null;
    if (lastExecute != null){
        lastExecuteDate = new Date(lastExecute.startDate);
    }

    var startBound = Parser.parseServerDate(task.Schedule.StartBoundary);
	/*console.log("start");
	console.log(task.Schedule.StartBoundary);
	console.log(startBound);*/
	startBound.setSeconds(0);
	
    var endBound = Parser.parseServerDate(task.Schedule.EndBoundary);
    if (endBound == null){
        endBound = new Date();
        endBound.setYear(9000);
    }
	endBound.setSeconds(59);
	/*console.log("end");
	console.log(task.Schedule.EndBoundary);
	console.log(endBound);*/
	
    var c = new Date();	
    if (c.getTime() >= startBound.getTime() && c.getTime() < endBound.getTime()){		
        
		if (task.baseExecuteDate == null){
			task.baseExecuteDate  = startBound.getTime();
			console.log("Task never run before, executing task");
			this.executeTask(task);
			return;
		}
		
        var repeatDuration = Parser.convertJsonTimespanToDate(task.Schedule.RepetitionPattern.RepeatDuration);
        var repeatInterval = Parser.convertJsonTimespanToDate(task.Schedule.RepetitionPattern.RepeatInterval);
        var baseExecuteDate = new Date(task.baseExecuteDate);

        var repeatEndDate = new Date(task.baseExecuteDate);
        repeatEndDate.setDate(repeatEndDate.getDate() + repeatDuration.Day);
        repeatEndDate.setHours(repeatEndDate.getHours() + repeatDuration.Hour);
        repeatEndDate.setMinutes(repeatEndDate.getMinutes() + repeatDuration.Minute);
		repeatEndDate.setSeconds(59);
		
        var nextExecute = new Date(lastExecuteDate.getTime());
        if (c.getTime() < repeatEndDate.getTime() && c.getTime() >= baseExecuteDate.getTime()){
            if (lastExecute != null){
                nextExecute.setMinutes(nextExecute.getMinutes() + repeatInterval.Minute);
                nextExecute.setSeconds(0);
                nextExecute.setMilliseconds(0);
            }
        }else{
            //Undefined = 0,Time_Trigger = 1,Daily_Trigger = 2,Weekly_Trigger = 3,Monthly_Trigger = 4,
            var daybuffer = 0;
            switch(task.Schedule.ScheduleTriggerType){
                case 2:
                    daybuffer = task.Schedule.DaysInterval * 1;
                    break;
                case 3:
                    daybuffer = (task.Schedule.WeeksInterval*1)*7;
                    break;
            }
            nextExecute.setDate(nextExecute.getDate() + daybuffer);
            nextExecute.setHours(startBound.getHours())
            nextExecute.setMinutes(startBound.getMinutes());
            nextExecute.setSeconds(0);
            nextExecute.setMilliseconds(0);
            task.baseExecuteDate = nextExecute.getTime();
        }
		console.log("----");
		console.log("next execute:" + nextExecute);
		console.log("last execute:" + lastExecuteDate);
        if (c.getTime() >= nextExecute.getTime()){
            this.executeTask(task);
        }
    }
}

AdvanceDeviceManagement.prototype.executeTaskOnDailyWeeklyMonthly = function(task){
	var lastExecuteHistory = this.getLastTaskHistory(task.Id);
    var lastExecuteDate = null;
    if (lastExecuteHistory != null){
        lastExecuteDate = new Date(lastExecuteHistory.startDate);
    }
	
	
	
	var startBoundary = Parser.parseServerDate(task.Schedule.StartBoundary,{treatUTCasLocal:true});
	startBoundary.setSeconds(0);
	startBoundary.setMilliseconds(0);
	var endBoundary = new Date();
	if (task.Schedule.EndBoundary == null){
		endBoundary.setYear(9000);
		endBoundary.setMinutes(0);
	}
	else{
		endBoundary = Parser.parseServerDate(task.Schedule.EndBoundary,{treatUTCasLocal:true});
	}
	endBoundary.setSeconds(59);
	endBoundary.setMilliseconds(0);
	var c = new Date();
	
    /*console.log('------');
    console.log('task id:' + task.Id);
    console.log('start boundary:' + startBoundary);
    console.log('end boundary:' + endBoundary);*/
	//if the current time is inside the task's start and end boundary
    if (c.getTime() >= startBoundary.getTime() && c.getTime() < endBoundary.getTime()){
		if (task.baseExecuteDate == null){			
			task.baseExecuteDate  = startBoundary.getTime();//base time for execution
			//this.executeTask(task);
			//return;
		}
		
		//retrieve task repetition pattern and end time
		var repeatDuration = Parser.convertJsonTimespanToDate(task.Schedule.RepetitionPattern.RepeatDuration);
        var repeatInterval = Parser.convertJsonTimespanToDate(task.Schedule.RepetitionPattern.RepeatInterval);
		
		var baseExecuteDate = new Date(task.baseExecuteDate);
		var repeatEndDate = new Date(task.baseExecuteDate);
        repeatEndDate.setDate(repeatEndDate.getDate() + repeatDuration.Day);
        repeatEndDate.setHours(repeatEndDate.getHours() + repeatDuration.Hour);
        repeatEndDate.setMinutes(repeatEndDate.getMinutes() + repeatDuration.Minute);
		repeatEndDate.setSeconds(59);
		
		var nextExecute = new Date(task.baseExecuteDate);
		var intervalTotalMinutes = 0;
		intervalTotalMinutes = repeatInterval.Minute;
		intervalTotalMinutes += repeatInterval.Hour * 60;
		intervalTotalMinutes += repeatInterval.Day * 24 * 60;
		var checkTime = new Date();
		checkTime.setSeconds(0);
		checkTime.setMilliseconds(0);
		if (checkTime.getTime() < repeatEndDate.getTime() && checkTime.getTime() >= baseExecuteDate.getTime()){
			//get next execute time inside the repetition period
			while(nextExecute.getTime() < checkTime.getTime()){
				nextExecute.setMinutes(nextExecute.getMinutes() + intervalTotalMinutes);
			}
			nextExecute.setSeconds(0);
			nextExecute.setMilliseconds(0);
			
			if (lastExecuteDate != null){			
				var lastExecuteCheck = new Date(lastExecuteDate.getTime());
				lastExecuteCheck.setSeconds(0);
				lastExecuteCheck.setMilliseconds(0);
				if (nextExecute.getTime() == lastExecuteCheck.getTime())
					nextExecute.setMinutes(nextExecute.getMinutes() + intervalTotalMinutes);
			}
            
        }else if (baseExecuteDate.getTime() < c.getTime()){
			//repetition period is done, get next start task time repetition
			
			
			//Run_Daily = 3,Run_Weekly = 4,Run_Monthly = 5
			var totalMinutes = 0;
			if (task.TaskScheduleType == 3){//daily
				totalMinutes = task.Schedule.DaysInterval * 24 * 60;
				var checkNextExecute = new Date(nextExecute.getTime());
				checkNextExecute.setSeconds(0);
				checkNextExecute.setMilliseconds(0);
				checkNextExecute.setMinutes(checkNextExecute.getMinutes() + totalMinutes);
				var finalTotalMinutes = totalMinutes;
				while (checkNextExecute.getTime() < c.getTime()){
					finalTotalMinutes += totalMinutes;
					checkNextExecute.setMinutes(checkNextExecute.getMinutes() + totalMinutes);
				}
				totalMinutes = finalTotalMinutes;
			} else if (task.TaskScheduleType == 4) {//weekly
				var tomorrow = new Date(c.getTime());
				if (tomorrow.getDate() == baseExecuteDate.getDate() && tomorrow.getMonth() == baseExecuteDate.getMonth() && tomorrow.getYear() == baseExecuteDate.getYear()){
					tomorrow.setDate(tomorrow.getDate() + 1);
				}
				
				var checkday = tomorrow.getDay(); //0 = Sunday -> 6 = Saturday
				var diffDay = -1;
				var taskBitwise = task.Schedule.DaysOfTheWeek;

				//get the next active day				
				//console.log('task bit wise day:' + task.Schedule.DaysOfTheWeek);
				var doCount = 0;
				while (true){
					if (doCount >= 14)
						break;
					
					var res = ((DaysOfTheWeek[checkday] & task.Schedule.DaysOfTheWeek) == DaysOfTheWeek[checkday]);
					//console.log(checkday);
					//console.log(res);
					if (res == true){
						//diffDay = doCount + 1;
						var d1 = nextExecute.getTime();
						var d2 = c.getTime();
						var diff = Math.floor((d2-d1)/(1000*60*60*24)) + 1;
						//console.log(diff);
						//console.log(doCount);
						diffDay = diff + doCount;
						break;
					}
					
					checkday++;

					if(checkday > 6)
						checkday = 0;

					doCount++;
				}
				//console.log('diff day:' + diffDay);
				if (diffDay == -1)
					diffDay = 7;

				totalMinutes = diffDay * 24 * 60;
			}
			//else if (task.TaskScheduleType == 5)//monthly
			
			if (totalMinutes != 0){
				nextExecute.setMinutes(nextExecute.getMinutes() + totalMinutes);
				nextExecute.setSeconds(0);
				nextExecute.setMilliseconds(0);
				task.baseExecuteDate = nextExecute.getTime();
			}
			
			
		}
        /*console.log("res:--");
		console.log("base execute:" + baseExecuteDate);
		console.log("next execute:" + nextExecute);
		console.log("previous execute:" + lastExecuteDate);
		console.log("repeat end:" + repeatEndDate);*/

		if (c.getTime() >= nextExecute.getTime()){
            if (lastExecuteDate != null) {
                var checkLastExecuteDate = new Date(lastExecuteDate.getTime());
                checkLastExecuteDate.setSeconds(0);
                checkLastExecuteDate.setMilliseconds(0);
                if (checkLastExecuteDate.getTime() == checkTime.getTime()) {
                    //console.log("skip execute. already run on this minute block")
                    return;
                }
            }
            this.executeTask(task);
        }
	}
}

/**
 *
 * @param {DeployedTask} task
 */
AdvanceDeviceManagement.prototype.executeTaskOnFrequency = function(task){
    var repeatInterval = Parser.convertJsonTimespanToDate(task.Schedule.RepetitionPattern.RepeatInterval);
    var lastExecute = this.getLastTaskHistory(task.Id);
    var lastExecuteDate = null;
	if (lastExecute != null) lastExecuteDate = new Date(lastExecute.startDate);
	
    /*if (lastExecute == null){
        var th = new TaskHistory();
        lastExecuteDate = Parser.parseServerDate(task.Schedule.StartBoundary);
        th.taskID = task.Id;
        th.startDate = lastExecuteDate.getTime();
        th.endDate = th.startDate;
        this.addTaskHistory(th);
    }
    else{
        lastExecuteDate = new Date(lastExecute.startDate);
    }*/

    var c = new Date();
    var nextExecute = new Date();
	if (lastExecuteDate != null){
		nextExecute = new Date(lastExecuteDate.getTime());
		nextExecute.setDate(nextExecute.getDate() + repeatInterval.Day);
		nextExecute.setHours(nextExecute.getHours() + repeatInterval.Hour);
		nextExecute.setMinutes(nextExecute.getMinutes() + repeatInterval.Minute);
		nextExecute.setSeconds(0);
		nextExecute.setMilliseconds(0);
	}
	
    if (lastExecuteDate == null || c.getTime() >= nextExecute.getTime()){
        this.executeTask(task);
    }
}
/**
 *
 * @param {Number} taskId
 * @returns {TaskHistory}
 */
AdvanceDeviceManagement.prototype.getLastTaskHistory = function(taskId){
    var res = null;
    var l = this.deployedTaskExecutionHistories.length;
    for (var i = 0 ; i < l ; i++){
        var th = this.deployedTaskExecutionHistories[i];
        if (th.taskID == taskId){
            res = th;
            break;
        }

    }

    return res;
}

/**
 *
 * @param {TaskHistory} taskHistory
 */
AdvanceDeviceManagement.prototype.addTaskHistory = function(taskHistory){
    var l = this.deployedTaskExecutionHistories.length;
    var found = false;
    for (var i = 0 ; i < l ; i++){
        var th = this.deployedTaskExecutionHistories[i];
        if (th.taskID == taskHistory.taskID){
            this.deployedTaskExecutionHistories[i] = taskHistory;
            found = true;
            break;
        }
    }
    if (found == false){
        this.deployedTaskExecutionHistories.push(taskHistory);
    }
}

/**
 *
 * @param {DeployedTask} task
 */
AdvanceDeviceManagement.prototype.executeTask = function(task){

    if (this.tasksExecuting[task.Id] != null){
        this.logger.log('info','[ADM] Task :' + task.Name + ' (Id:' + task.Id + ') is running. Skip execute.');
        return;
    }




    
    var definitions = task.Definitions;
    var l = definitions.length;
    var executed = false;
    for (var i = 0 ; i < l ; i++){
        var definition = definitions[i];

        if(this.resolveDeviceType(definition.ForDevice) == this.deviceType){
            var taskCommand = definition.TaskCommand;
			
			if (taskCommand == null)
				continue;
				
			executed = true;			
            // Undefined = 0,Url = 1,Shell = 3,RS232 = 4,Socket = 5,
            switch(taskCommand.Protocol){
                case 1:
					this.logger.log('info','[ADM] Executing task (URL) :' + task.Name + ' (Id:' + task.Id + ')');
                    this.executeProtocolUrl(task,i);
                    break;
                case 3:
					this.logger.log('info','[ADM] Executing task (shell) :' + task.Name + ' (Id:' + task.Id + ')');
                    this.executeProtocolShell(task,i);
                    break;
                case 4:
					this.logger.log('info','[ADM] Executing task (RS232) :' + task.Name + ' (Id:' + task.Id + ')');
                    this.executeRS232Protocol(task,i);
                    break;
                case 5:
					this.logger.log('info','[ADM] Executing task (socket) :' + task.Name + ' (Id:' + task.Id + ')');
                    this.executeProtocolSocket(task,i);
                    break;
                default:
                    //this.logger.log('info','[ADM] Task not executed. No supported protocol ('+taskCommand.Protocol+')')
                    break;
            }
            break;
        }
    }
    if (executed == false){
        //this.logger.log('info','[ADM] Task not executed. No definition for current device type ('+this.deviceType+')');
    }

}

AdvanceDeviceManagement.prototype.executeRS232Protocol = function(task,definitionIndex){
    this.tasksExecuting[task.Id] = task;
    var properties =task.Definitions[definitionIndex].TaskCommand.Properties;
    var replaceParams = this.retrieveReplaceParametersDict(task,definitionIndex);

    var payload = '';
    var l = properties.length;
    for (var i = 0 ; i < l ; i++){
        var p = properties[i];
        switch(p.Key.toLowerCase()){
            case 'body':
                payload = p.Value;
                break;
        }
    }

    var taskstate = new TaskState();
    taskstate.task = task;
    taskstate.startdate = (task.virtualExecuteDate != null)?new Date(task.virtualExecuteDate):new Date();
	task.virtualExecuteDate = null;
    taskstate.activeDefinitionIndex = definitionIndex;
    var desc = (task.Description == null)?"":task.Description;
    if (desc.toLowerCase().indexOf("{{savetaskimmediately}}") > -1)
        taskstate.immediatesave = true;
    if (payload != ''){

        for ( var key in replaceParams)
            payload = payload.split(key).join(replaceParams[key]);

        if (payload.toLowerCase().indexOf('http://') > -1 || payload.toLowerCase().indexOf('https://') > -1){
            
            var options = RequestUtil.createOptionsObject();
		    options.url = payload;
		    options.failCallback = EventHandler.CreateEventHandler(this,this.onReceiveExecuteProtocolUrlHandler,taskstate);
		    options.successCallback = EventHandler.CreateEventHandler(this,this.onReceiveErrorExecuteProtocolUrlHandler,taskstate);

			RequestUtil.makeRequest(options);
        }
        else{

            exec(payload,EventHandler.CreateEventHandler(this,this.onReceiveExecuteProtocolShellHandler,taskstate));
        }
    }
    else{
        taskstate.success = false;
        taskstate.response = 'body is empty';
        this.onTaskExecuted(taskstate);
    }

}

AdvanceDeviceManagement.prototype.executeProtocolSocket = function(task,definitionIndex){
    this.tasksExecuting[task.Id] = task;
    var properties =task.Definitions[definitionIndex].TaskCommand.Properties;
    var replaceParams = this.retrieveReplaceParametersDict(task,definitionIndex);

    var port = -1;
    var host = '';
    var payload = '';
    var url = '';

    var l = properties.length;
    for (var i = 0 ; i < l ; i++){
        var p = properties[i];
        switch(p.Key.toLowerCase()){
            case 'url.url':
                url = p.Value;
                break;
            case 'body':
                payload = p.Value;
                break;
        }
    }

    host = host.replace('http://','').replace('https://','').replace('HTTP://','').replace('HTTPS://','');
    var ss = host.split(':');
    host = ss[0];
    if (ss.length > 1)
        port = parseInt(ss[1],10);
    else
        port = '';

    var taskstate = new TaskState();
    taskstate.task = task;
    taskstate.startdate = (task.virtualExecuteDate != null)?new Date(task.virtualExecuteDate):new Date();
	task.virtualExecuteDate = null;
    taskstate.activeDefinitionIndex = definitionIndex;
    var desc = (task.Description == null)?"":task.Description;
    if (desc.toLowerCase().indexOf("{{savetaskimmediately}}") > -1)
        taskstate.immediatesave = true;
    if (host != '' && port != -1){
        for ( var key in replaceParams){
            host = host.split(key).join(replaceParams[key]);
            port = port.split(key).join(replaceParams[key]);
            payload = payload.split(key).join(replaceParams[key]);
        }
        var options = {
            port : port,
            host : host
        };
        var client = net.connect(options);
        taskstate.handler = client;
        taskstate.payload = payload;
        client.on('connect',EventHandler.CreateEventHandler(this,this.onSocketConnectedHandler,taskstate));
        client.on('data',EventHandler.CreateEventHandler(this,this.onSocketDataReceivedHandler,taskstate));
        client.on('end',EventHandler.CreateEventHandler(this,this.onSocketClosedHandler,taskstate));
    }else{
        taskstate.success = false;
        taskstate.response = 'port/host is empty';
        this.onTaskExecuted(taskstate);
    }
}

AdvanceDeviceManagement.prototype.onSocketConnectedHandler = function(calle,taskstate){
    calle.onSocketConnected(taskstate);
}

/**
 * @param {TaskState} taskstate
 */
AdvanceDeviceManagement.prototype.onSocketConnected = function(taskstate){
    var client = taskstate.handler;
    client.write(taskstate.payload);
}

AdvanceDeviceManagement.prototype.onSocketDataReceivedHandler = function(calle,taskstate,data){
    calle.onSocketDataReceived(taskstate,data);
}

/**
 * @param {TaskState} taskstate
 * @param data
 */
AdvanceDeviceManagement.prototype.onSocketDataReceived = function(taskstate,data){
    taskstate.response = data;
}

AdvanceDeviceManagement.prototype.onSocketClosedHandler = function(calle,taskstate){
    calle.onSocketClosed(taskstate);
}

AdvanceDeviceManagement.prototype.onSocketClosed = function(taskstate){
    this.onTaskExecuted(taskstate);
}

/**
 *
 * @param {DeployedTask} task
 */
AdvanceDeviceManagement.prototype.executeProtocolUrl = function(task,definitionIndex){
    this.tasksExecuting[task.Id] = task;
    var properties =task.Definitions[definitionIndex].TaskCommand.Properties;
    var url = '';
    var method = '';
    var header = '';
    var body = '';
    var replaceParams = this.retrieveReplaceParametersDict(task,definitionIndex);

    var l = properties.length;
    for (var i = 0 ; i < l ; i++){
        var p = properties[i];
        switch(p.Key.toLowerCase()){
            case 'url.url':
                url = p.Value;
                break;
            case 'url.method':
                method = p.Value;
                break;
            case 'url.header':
                header = p.Value;
                break;
            case 'body':
                body = p.Value;
                break;
        }
    }

    var taskstate = new TaskState();
    taskstate.task = task;
    taskstate.startdate = (task.virtualExecuteDate != null)?new Date(task.virtualExecuteDate):new Date();
	task.virtualExecuteDate = null;
    taskstate.activeDefinitionIndex = definitionIndex;
    var desc = (task.Description == null)?"":task.Description;
    if (desc.toLowerCase().indexOf("{{savetaskimmediately}}") > -1)
        taskstate.immediatesave = true;

    if (url != ''){
		if (header == null)
			header = '';
		if (body == null)
			body = '';
		if (method == null)
			method = '';
        for ( var key in replaceParams){
            url = url.split(key).join(replaceParams[key]);
            method = method.split(key).join(replaceParams[key]);
            header = header.split(key).join(replaceParams[key]);
            body = body.split(key).join(replaceParams[key]);

        }

        /*var path = url.replace('http://','').replace('https://','').replace('HTTP://','').replace('HTTPS://','');
        var index = path.indexOf('/');
        var tempurl = path;
        var port = '';
        if (index > -1){
            tempurl = path.substring(0,index);
            path = path.replace(tempurl,'');
            tempurl = tempurl.replace('/','');
            if (tempurl.indexOf(':') > -1){
                port = tempurl.split(':')[1];
                tempurl = tempurl.split(':')[0];
            }
        }*/

        if (header.length > 0){
            var resheader = {};
            var splits = header.split(';');
            l = splits.length;
            for (var i = 0 ; i < l ; i++){
                var delim1 = splits[i].indexOf('=');
                var delim2 = splits[i].indexOf(':');

                if (delim1 == -1 && delim2 == -1)
                    continue;

                var delimiter = '=';
                if (delim1 == -1 && delim2 != -1)
                    delimiter = ':';
                else if (delim1 != -1 && delim2 == -1)
                    delimiter = '=';
                else if (delim1 != -1 && delim2 != -1)
                    delimiter = (delim1 < delim2)?'=':':';                

                var kvps = splits[i].split(delimiter);
                if (kvps.length >= 2){
                    resheader[kvps[0]] = kvps.slice(1,kvps.length).join(delimiter);
                }
            }
            header = resheader;
        }

		//var prot = (url.toLowerCase().indexOf('https://') > -1)?"https://":"http://";
		//var Curl = prot + tempurl + path;

		method = method.toUpperCase();

	 	var options = RequestUtil.createOptionsObject();
	    options.url = url;
	    options.method = method;
	    options.headers = header;
	    options.failCallback = EventHandler.CreateEventHandler(this,this.onReceiveErrorExecuteProtocolUrlHandler,taskstate);
	    options.successCallback = EventHandler.CreateEventHandler(this,this.onReceiveExecuteProtocolUrlHandler, taskstate);
	    if (body != null && body.length > 0 && options.method.toLowerCase() != 'get'){
			options.body = body;
		}
		//console.log(options);
	   	RequestUtil.makeRequest(options);
    }
    else{
        taskstate.success = false;
        taskstate.response = 'url.url is empty';
        this.onTaskExecuted(taskstate);
    }

}


/**
 *
 * @param calle
 * @param {TaskState} state
 * @param err
 */
AdvanceDeviceManagement.prototype.onReceiveErrorExecuteProtocolUrlHandler = function(calle,state,err){
    state.response = 'Receive Error:' + err.message;
    calle.onTaskExecuted(state);
}

AdvanceDeviceManagement.prototype.onReceiveExecuteProtocolUrlHandler = function(calle,state,res,body){
    calle.onReceiveExecuteProtocolUrl(state,res,body);
}

/**
 *
 * @param {TaskState} state
 * @param res
 */
AdvanceDeviceManagement.prototype.onReceiveExecuteProtocolUrl = function(state,res,body){
    var owner = this;
    if((res.statusCode >= 200 && res.statusCode < 300) || res.statusCode === 304){
        state.success = true;
        var data = '';
        data += body.toString();
		state.response = data; 
		owner.onTaskExecuted(state);
    }else{
        state.success = false;
        state.response = res.statusCode;
        owner.onTaskExecuted(state);
    }

}

/**
 *
 * @param {DeployedTask} task
 */
AdvanceDeviceManagement.prototype.executeProtocolShell = function(task,definitionIndex){
    this.tasksExecuting[task.Id] = task;
    var properties =task.Definitions[definitionIndex].TaskCommand.Properties;
    var command = '';
    var replaceParams = this.retrieveReplaceParametersDict(task,definitionIndex);
    var l = properties.length;
    for (var i = 0 ; i < l ; i++){
        var p = properties[i];
        if (p.Key.toLowerCase() == 'body'){
            command = p.Value;
            break;
        }
    }

    var taskstate = new TaskState();
    taskstate.task = task;
    taskstate.startdate = (task.virtualExecuteDate != null)?new Date(task.virtualExecuteDate):new Date();
	task.virtualExecuteDate = null;
    taskstate.activeDefinitionIndex = definitionIndex;
    var desc = (task.Description == null)?"":task.Description;
    if (desc.toLowerCase().indexOf("{{savetaskimmediately}}") > -1)
        taskstate.immediatesave = true;
    if (command != ''){
        for ( var key in replaceParams)
            command = command.split(key).join(replaceParams[key]);//command = command.replace(key,replaceParams[key]);
		
		switch(this.deviceType){
			case 'edge340' :
			case 'edge300' :
			case 'dmp4310' :
			case 'dmp4400' :
				command = command.split('\r\n').join('\n').split('\r').join('\n');
				break;
		}
		
        exec(command,EventHandler.CreateEventHandler(this,this.onReceiveExecuteProtocolShellHandler,taskstate));
    }
    else{
        taskstate.success = false;
        taskstate.response = 'command is empty';
        this.onTaskExecuted(taskstate);
    }
}

/**
 * @param {AdvanceDeviceManagement} calle
 * @param {TaskState} taskExecution
 * @param error
 * @param stdout
 * @param stderr
 */
AdvanceDeviceManagement.prototype.onReceiveExecuteProtocolShellHandler = function(calle,state,error,stdout,stderr){
    calle.onReceiveExecuteProtocolShell(state,error,stdout,stderr);
}

/**
 *
 * @param {TaskState} taskExecution
 * @param error
 * @param stdout
 * @param stderr
 */
AdvanceDeviceManagement.prototype.onReceiveExecuteProtocolShell = function(state,error,stdout,stderr){
    if (error != null){
        state.response = stderr;
        state.success = false;
    }
    else{
        state.success = true;
        state.response = stdout;
    }
    this.onTaskExecuted(state);
}

/**
 *
 * @param {DeployedTask} task
 */
AdvanceDeviceManagement.prototype.retrieveReplaceParametersDict = function(task,definitionIndex){

    var commonParameters = task.Parameters;
    var definitionSpecificParameters = task.Definitions[definitionIndex].Parameters;

    var l = commonParameters.length;
    var replaceParams = [];
    for (var i = 0 ; i < l ; i++){
        var p = commonParameters[i];
		var placeholder = p.Placeholder;
		var paramvalue = p.ParamValue;
		
		placeholder = placeholder.replace('{{','').replace('}}','').replace('({','').replace('})','');
		
        replaceParams['{{' + placeholder + '}}'] = replaceParams['({' + placeholder + '})'] = paramvalue;
    }

    l = definitionSpecificParameters.length;
    for (var i = 0 ; i < l ; i++){
        var p = definitionSpecificParameters[i];
		
        var placeholder = p.Placeholder;
		var paramvalue = p.ParamValue;
		
		placeholder = placeholder.replace('{{','').replace('}}','').replace('({','').replace('})','');
		
        replaceParams['{{' + placeholder + '}}'] = replaceParams['({' + placeholder + '})'] = paramvalue;
    }
	
	for ( var key in this.environmentVars ){
		replaceParams['{{' + key + '}}'] = replaceParams['({' + key + '})'] = this.environmentVars[key];
	}
	
	replaceParams['({ENV_TASK_ID})'] = replaceParams['{{ENV_TASK_ID}}'] = task.Id;

    return replaceParams;
}

/**
 *
 * @param {TaskState} state
 */
AdvanceDeviceManagement.prototype.onTaskExecuted = function(state){
    var th = new TaskHistory();
    var c = new Date();
    th.taskID = state.task.Id;
    th.startDate = state.startdate.getTime();
    th.endDate = c.getTime();
    this.addTaskHistory(th);

    fs.writeFile(this.deployedTaskExecutedHistoryPath,JSON.stringify(this.deployedTaskExecutionHistories));

    var owner = this;

    var TaskRequest = {
        Response : { Raw : state.response, DeployedTaskId : state.task.Id , ClientGuid : owner.playerID },
        DeviceType : state.task.Definitions[state.activeDefinitionIndex].ForDevice
    }

    var postdata = JSON.stringify(TaskRequest);

	var options = RequestUtil.createOptionsObject();
    options.url = owner.appspaceURL + '/api/v1/core/tasks/response';
    options.method = 'POST';
    //options.headers = {'Content-Type' : 'application/json'};
    options.headers['Content-Type'] = 'application/json';
    options.body = postdata;
    options.failCallback = EventHandler.CreateEventHandler(this, this.onErrorOnTaskExecuted);
	options.successCallback = EventHandler.CreateEventHandler(this, this.onTaskExecutedResponseSuccess);
   	RequestUtil.makeRequest(options);
    
	this.logger.log('info','[ADM] Task ' + state.task.Name + ' (Id:'+ state.task.Id +') executed.')
    this.logger.log('info','[ADM] Posting "' + postdata + '" to "'+ options.url + '"');	
	
    this.tasksExecuting[state.task.Id] = null;
    if (state.immediatesave == true)
        this.transferFilesFromCacheToPermanent();
}

AdvanceDeviceManagement.prototype.onTaskExecutedResponseSuccess = function(calle,res,body){
	//console.log(body);
	//console.log(res);
}

AdvanceDeviceManagement.prototype.onErrorOnTaskExecuted = function (calle, error){
	//console.log('Error on posting task response:' + error);
}

AdvanceDeviceManagement.prototype.resolveDeviceType = function(deviceType) {
    var res = '';

    switch(deviceType.toLowerCase()){
        case 'dmp-4310':
        case 'dmp4310':
        case 'dmp 4310':
            res = 'dmp4310';
            break;
        case 'dmp-4400':
        case 'dmp4400':
        case 'dmp 4400':
            res = 'dmp4400';
            break;
        case 'edge 300':
        case 'edge300':
        case 'edge-300':
        case 'e300':
            res = 'edge300';
            break;
		case 'edge 340':
        case 'edge340':
        case 'edge-340':
        case 'e340':
            res = 'edge340';
            break;
    }
    return res;
}

var DaysOfTheWeek = new Object();
DaysOfTheWeek[0] = (0x1);
DaysOfTheWeek[1] = (0x2);
DaysOfTheWeek[2] = (0x4);
DaysOfTheWeek[3] = (0x8);
DaysOfTheWeek[4] = (0x10);
DaysOfTheWeek[5] = (0x20);
DaysOfTheWeek[6] = (0x40);
DaysOfTheWeek[7] = (0x7F);

var net = require('net');
var http = require('http');
var https = require('https');
var exec = require('child_process').exec;
var fs = require('fs');
var DeployedTask = require('../adm/deployedtask');
var TaskHistory = require('../adm/taskhistory');
var EventHandler = require('../util/eventhandler');
var RequestUtil = require('../util/requestUtil');
var TaskQueue = require('../adm/taskqueue');
var Parser = require('../util/parser');
var TaskState = require('../adm/taskstate');
var AppspaceLogger = require("../util/log");
var fsextra = require('fs.extra');
module.exports = AdvanceDeviceManagement;