var Log = require("log");
/**
 *
 * @param {String} workingPath
 * @constructor
 */
function AppspaceLogger(workingPath){
    var fs = require('fs');
    this.workingPath = workingPath;
    this.tempLog = '/tmp/appspace/appspacenode.log';
    this.permanentLog = this.workingPath + '/Logs/appspacenode.log';
    /** @type {Log} */
    this.logger = new Log('info', fs.createWriteStream(this.tempLog, { flags: "a+" }));

    this.intervalForBackup = 5 * 60 * 1000;
    //this.intervalForBackup = 5000;
    this.lastBackup = new Date();
    this.maxFileSize = 30240000;
    setTimeout(EventHandler.CreateEventHandler(this,this.onCheckTimerHandler),5000);
}

AppspaceLogger.prototype.onCheckTimerHandler = function(calle){
    calle.onCheckTimer();
}

AppspaceLogger.prototype.onCheckTimer = function(){
    var c = new Date();

    if (c.getTime() - this.lastBackup.getTime() >= this.intervalForBackup){
        this.lastBackup = new Date();
        this.doBackup();
    }

    setTimeout(EventHandler.CreateEventHandler(this,this.onCheckTimerHandler),5000);
}

AppspaceLogger.prototype.log = function(type,message){
    var d = new Date();
    console.log(d.getFullYear() + '/' + (d.getMonth() + 1) + '/' + d.getDate() + ' ' + d.getHours() + ':' + d.getMinutes() + ':' + d.getSeconds() + '.' + d.getMilliseconds() + ' : ' + type + ' ' + message);

    switch(type.toLowerCase()){
        case "info":
            this.logger.info(message);
            break;
        case "error":
            this.logger.error(message);
            break;
        case "alert":
            this.logger.alert(message);
            break;
        case "warning":
            this.logger.warning(message);
            break;
    }

}

AppspaceLogger.prototype.doBackup = function(){
    var fs = require('fs');
    fs.createReadStream(this.tempLog).pipe(fs.createWriteStream(this.permanentLog));
    var owner = this;
    fs.stat(this.tempLog,function(err,stats){
        var filesize = parseInt(stats.size.toString(),10);
        var maxsize = owner.maxFileSize;
        if (filesize > maxsize){
            fs.writeFile(owner.tempLog,'',function(err){owner.logger.info('[Logger] Log file is emptied after ' + owner.maxFileSize + ' bytes');});
        }
    });
}

var EventHandler = require('../util/eventhandler');
module.exports = AppspaceLogger;