/**
 * Created by syed.husain on 27/03/14.
 */
var IRProxyService = require('./services/irproxy/main.js').IRPRoxyService;
var AppspaceService = require('./services/general/main.js').AppspaceGeneralServices;
var AppspacePMService = require('./services/prioritymessaging/main.js').AppspacePMService;
var RemoteSocketService = require('./services/socket/main.js').RemoteSocket;
var FileService = require('./services/fileserver/main.js').FileServer;
var DeviceType = require('./util/devicetype.js');
var LogService = require('./services/logservice/main').AppspaceLogService;

function AppspaceNodeServices(){
    this.irEnabled = false;
    this.appspaceServiceEnabled = false;
    this.remoteSocketServiceEnabled = false;
    this.fileServiceEnabled = false;
	this.pmServiceEnabled = false;
    this.irService = null;
    this.appspaceService = null;
    this.remoteSocketService = null;
	this.pmService = null;
    this.deviceType = null;
	this.logService = null;
}

AppspaceNodeServices.prototype.start = function(){
    this.loadconfig();
}

AppspaceNodeServices.prototype.loadconfig = function(){

    var arguments = process.argv;
    var l = arguments.length;
    var tempDeviceType = '';

    for (var i = 2 ; i < l ; i++){
        var use = true;
        switch(arguments[i].toLowerCase()){
            case '-enableirproxyservice':
                this.irEnabled = true;
                break;
            case '-enableappspaceservice':
                this.appspaceServiceEnabled = true;
                break;
            case '-enableremotesocketservice':
                this.remoteSocketServiceEnabled = true;
                break;
            case '-enablefileserverservice':
                this.fileServiceEnabled = true;
                false;
			case '-enablepm':
				this.pmServiceEnabled = true;
				false;
            default:
                use = false;
                break;
        }

        if (use == false){
            if (arguments[i].toLowerCase().indexOf('-devicetype=') >= 0){
                tempDeviceType = arguments[i].split('-devicetype=').join('').toLowerCase();
            }
        }
    }

    switch(tempDeviceType.toLowerCase()){
        case "dmp-4310":
        case "dmp4310":
        case "dmp 4310":
            this.deviceType = DeviceType.DMP4310;
            break;
        case "dmp-4400":
        case "dmp4400":
        case "dmp 4400":
            this.deviceType = DeviceType.DMP4400;
            break;
        case "desktop":
        case "pc":
            this.deviceType = DeviceType.PC;
            break;
        case "edge 300":
        case "edge-300":
        case "edge300":
        case "e300":
            this.deviceType = DeviceType.Edge300;
            break;
        case "edge 340":
        case "edge-340":
        case "edge340":
        case "e340":
            this.deviceType = DeviceType.Edge340;
            break;
        case "test":
            this.deviceType = DeviceType.DEBUG;
            break;
    }

    console.log('config loaded:');
    console.log('IR Proxy Service Enabled? ' + this.irEnabled);
    console.log('Appspace Service Enabled? ' + this.appspaceServiceEnabled);
    console.log('Appspace Remote Socket Service Enabled? ' + this.remoteSocketServiceEnabled);
	console.log('Appspace Priority Messagings Service Enabled? ' + this.pmServiceEnabled);
    console.log('Device Type? ' + this.deviceType);
    this.loadServices();
}

AppspaceNodeServices.prototype.loadServices = function(){

	this.logService = new LogService();
	this.logService.setConfig({deviceType:this.deviceType});
	this.logService.start();
	console.log('log service started');
	
    if (this.irEnabled == true && this.deviceType == DeviceType.Edge340){
        this.irService = new IRProxyService();
        this.irService.start();
        console.log('IR Proxy Service started');
    }

    if (this.remoteSocketServiceEnabled == true){
        this.remoteSocketService = new RemoteSocketService();
        console.log('Appspace Remote Socket Service initialized');
    }

    if (this.fileServiceEnabled == true){
        this.fileService = new FileService();
        this.fileService.deviceType = this.deviceType;
        console.log('Appspace File Server Service Initialized')
        this.fileService.start();
    }

	if (this.pmServiceEnabled == true){
		this.pmService = new AppspacePMService();
		this.pmService.setConfig({deviceType:this.deviceType});
		this.pmService.start();
	}
	
    if (this.appspaceServiceEnabled == true){
        this.appspaceService = new AppspaceService();
        this.appspaceService.setConfig({deviceType:this.deviceType});

        this.appspaceService.start();
        this.appspaceService.irService = this.irService;
        this.appspaceService.remoteSocketService = this.remoteSocketService;
        this.appspaceService.fileServerService = this.fileService;
		this.appspaceService.pmService = this.pmService;
        console.log('Appspace Service started');
    }
}



var service = new AppspaceNodeServices();
service.start();