/**
 * Created by syed.husain on 8/28/2014.
 */

var http = require('http');
var fs = require('fs');
var url = require('url');
var DeviceType = require('../../util/devicetype.js');
var EventHandler = require('../../util/eventhandler.js');
function AppspaceFileServer(){
    this.remotePort = 60108;
    this.remoteServer = null;
    this.isStarted = false;
    this.connectedWebClients = new Object();
    this.deviceType = DeviceType.DEBUG;
    this.subscribers = {};
}

AppspaceFileServer.prototype.startConnectionHandler = function(calle){
    calle.start();
}

AppspaceFileServer.prototype.start = function(){
    console.log('file server started');
    var calle = this;
    this.isStarted = true;
    var basePath = '';
    switch(this.deviceType){
        case DeviceType.DMP4310:
            basePath = '/tmp/ftproot/usb_1/';
            break;
        case DeviceType.DMP4400:
            basePath = '/tmp/ftproot/usb_1/';
            break;
        case DeviceType.Edge300:
        case DeviceType.Edge340:
            basePath = '/apps/nexus2/';
            break;
        case DeviceType.DEBUG:
        case DeviceType.PC:
            basePath = 'c:/fastnetv2';
            break;
    }
    this.remoteServer = http.createServer(function(request,response){
        var uri = url.parse(request.url).pathname;
        var filename = basePath + uri.split('%20').join(' ');
        var conName = request.socket.remoteAddress + ' ' + request.socket.remotePort;
        calle.subscribers[conName] = request;

        if (request.url.toLowerCase().indexOf('closeappspacefileserver') >= 0) {
            response.writeHead(200, {"Content-Type": "text/plain"});
            response.write('closing server');
            response.end();
            calle.stop();
            return;
        }

        fs.exists(filename,function(exists){
            if (!exists){
                response.writeHead(404, {"Content-Type": "text/plain"});
                response.write("404 Not Found\n");
                response.end();
                return;
            }

            if (fs.statSync(filename).isDirectory()) filename += '/index.html';

            fs.readFile(filename, "binary", function(err, file) {
                if(err) {
                    response.writeHead(500, {"Content-Type": "text/plain"});
                    response.write("Unable to serve "+ request.url + "\n");
                    response.end();
                    return;
                }

                response.writeHead(200);
                response.write(file, "binary");
                response.end();
            });
        });

    }).listen(this.remotePort,"127.0.0.1");

    this.remoteServer.addListener('close',function(){
        console.log('server closed. Reopening in 2 seconds')
        setTimeout(EventHandler.CreateEventHandler(calle,calle.startConnectionHandler),2000);
    });
}

AppspaceFileServer.prototype.stopConnectionHandler = function(calle){
    calle.stop();
}

AppspaceFileServer.prototype.stop = function(){
    console.log('file server stopped');
    for (var key in this.subscribers) {
        var request = this.subscribers[key];
        if (request != null && request.connection != null) {
            request.connection.destroy();
            this.subscribers[key] = null;
            delete this.subscribers[key];
            console.log('deleting ' + key);
        }
    }
    this.isStarted = false;
    if (this.remoteServer != null)
        this.remoteServer.close();
}

exports.FileServer = AppspaceFileServer;
