/**
 * Created by syed.husain on 014 14 04 2014.
 */
var EventHandler = require('../../util/eventhandler.js');
var ws = require('ws');
var DeviceType = require('../../util/devicetype.js');
var fs = require('fs');
var os = require('os');

function AppspaceGeneralServices(){
    //subscribers
    this.irWebSubscribers = {};
    this.socketWebSubscribers = {};
    this.pmSubscribers = {};
    
    this.remotePort = 61000;
    this.remoteServer = null;

    this.deviceType = DeviceType.Edge340;
    this.basePath = '/apps/nexus2';

    this.nodeid = '';
    this.appspaceip = '';
    this.localip = '';

    this.irService = null;
    this.remoteSocketService = null;
    this.fileServerService = null;
    this.pmService = null;
    
    this.onIRReceivedEventHandler = null;
    this.onRemoteSocketReceivedEventHandler = null;
    this.onPMReceivedEventHandler = null;
    
    this.pmData = '';
}

AppspaceGeneralServices.prototype.start = function(){
    this.onIRReceivedEventHandler = EventHandler.CreateEventHandler(this,this.onIRReceivedHandler);
    this.onRemoteSocketReceivedEventHandler = EventHandler.CreateEventHandler(this,this.onRemoteSocketReceivedHandler);
    this.onPMReceivedEventHandler = EventHandler.CreateEventHandler(this,this.onPMReceivedHandler);
    
    switch(this.deviceType){
        case DeviceType.Edge300:
        case DeviceType.Edge340:
            this.basePath = '/apps/nexus2';
            break;
        case DeviceType.DMP4310:
        case DeviceType.DMP4400:
            this.basePath = '/tmp/ftproot/usb_1';
            break;
    }

    this.loadFileConfig();
};

AppspaceGeneralServices.prototype.setConfig = function(config){
    this.deviceType = config.deviceType;
};

AppspaceGeneralServices.prototype.stop = function(){
    this.onIRReceivedEventHandler = null;
    if (this.remoteServer !== null){
        this.remoteServer.close();
    }
};

AppspaceGeneralServices.prototype.loadFileConfig = function(){
    var nodefile = this.basePath +'/node.dat';
    fs.readFile(nodefile,EventHandler.CreateEventHandler(this,this.onFileConfigReadHandler));
};

AppspaceGeneralServices.prototype.onFileConfigReadHandler = function(calle,err,data){
    if (err === null){
        data = data.toString();
        var splitter = (data.indexOf('\r\n') === -1)?'\n':'\r\n';
        var lines = data.split(splitter);
        calle.onFileConfigRead(lines);
    }
    else{
        calle.startServices();
    }
};

AppspaceGeneralServices.prototype.onFileConfigRead = function(lines){
    if (lines.length > 0){
        this.nodeid = lines[0];
        this.appspaceip = lines[2];
    }
    this.localip = '';
    var ifaces=os.networkInterfaces();
    var calle = this;
    for (var dev in ifaces) {
        var alias=0;

        ifaces[dev].forEach(function(details){
            if (details.family === 'IPv4') {
                var al = dev+(alias?':'+alias:'');
                if (al.toLowerCase().indexOf('lo') === -1) {
                    calle.localip = details.address;
                }
                ++alias;
            }
        });
    }

    this.startServices();
};

AppspaceGeneralServices.prototype.startServices = function(){

    if (this.irService !== null){
        this.irService.subscribe(this.onIRReceivedEventHandler);
    }

    if (this.remoteSocketService !== null){
        this.remoteSocketService.subscribe(this.onRemoteSocketReceivedEventHandler);
    }

    if (this.pmService !== null){
        this.pmService.subscribe(this.onPMReceivedEventHandler);
    }
    
    this.remoteServer = new ws.Server({port:this.remotePort,host:'127.0.0.1'});
    this.remoteServer.on('connection',EventHandler.CreateEventHandler(this,this.onConnectionEstablishedHandler));

}

AppspaceGeneralServices.prototype.onConnectionEstablishedHandler = function(calle,connection){
    calle.onConnectionEstablished(connection);
}

AppspaceGeneralServices.prototype.onConnectionEstablished = function(connection){
    var calle = this;
    var connectionName = connection._socket.remoteAddress + ":" + connection._socket.remotePort;
    connection.name = connectionName;
    console.log('connection established:' + connection.name);

    connection.on('close',function(code,reason){
        calle.removeWebSubscribers(connection.name);
    });

    var request = connection.upgradeReq.url;
    if (request.indexOf('/') >= 0)
        request = request.split('/').join('');

    if (request.indexOf('?') >= 0)
        request = request.split('?').join('');

    var options = request.split('&');
    var l = options.length;
    for (var i = 0; i < l ; i++){
        var key = options[i];
        if (key.toLowerCase().indexOf('listenir') >= 0){
            this.irWebSubscribers[connection.name] = connection;
        }
        else if (key.toLowerCase().indexOf('listensocket') >= 0){
            this.socketWebSubscribers[connection.name] = connection;
        }
        else if (key.toLowerCase().indexOf('sendhealth') >= 0){
            this.sendHealth(connection);
        }
        else if (key.toLowerCase().indexOf('listenpm') >= 0){
            this.pmSubscribers[connection.name] = connection;
            if (this.pmData !== ''){
				console.log('sending data to new subscriber');
                connection.send(this.pmData);
            }
        }
        else if (key.toLowerCase().indexOf('enableremotesocket') >= 0 || key.toLowerCase().indexOf('disableremotesocket') >= 0){
            if (key.toLowerCase().indexOf('enableremotesocket') >= 0 && this.remoteSocketService != null && this.remoteSocketService.isStarted == false){
                this.remoteSocketService.start();
            }
            else if (key.toLowerCase().indexOf('disableremotesocket') >= 0 && this.remoteSocketService != null && this.remoteSocketService.isStarted == true){
                this.remoteSocketService.stop();
            }
            connection.close();
        }
        else if (key.toLowerCase().indexOf('enablefileserver') >= 0 || key.toLowerCase().indexOf('disablefileserver') >= 0){
            if (key.toLowerCase().indexOf('enablefileserver') >= 0 && this.fileServerService != null && this.fileServerService.isStarted == false){
                this.fileServerService.start();
            }else if (key.toLowerCase().indexOf('disablefileserver') >= 0 && this.fileServerService != null && this.fileServerService.isStarted == true){
                this.fileServerService.stop();
            }
            connection.close();
        }
        else{
            connection.close();
        }
    }
}

AppspaceGeneralServices.prototype.sendHealth = function(connection){
    console.log('sending health status to Appspace');
    if (this.nodeid != '' && this.appspaceip != ''){
        var url = this.appspaceip + '/medianetintegrationservice/httpnodeservices.ashx?a=updatestatus&nodeid=' + this.nodeid + '&nodeip=' + this.localip + '&nodestatus=Online';
        console.log('calling ' + url);
        var protocol = (this.appspaceip.indexOf('https') == -1)?require('http'):require('https');
        protocol.get(url,function(res){
            res.on('data', function (chunk) {
                chunk = chunk.toString();
                console.log('res from ' + url);console.log(chunk);connection.send(chunk);connection.close();
            });

        }).on('error',function(e){
			console.log('error when sending health');
			connection.send('error when sending health');
			connection.close();
		});
    }
    else {
        connection.send('error. no node id or appspapce ip');
		connection.close();
    }
}

AppspaceGeneralServices.prototype.removeWebSubscribers = function(name){
    console.log('removing subscriber ' + name + ' from:');
    if (this.irWebSubscribers[name] != null){
        console.log('IR');
        this.irWebSubscribers[name] = null;
        delete this.irWebSubscribers[name];
    }

    if (this.socketWebSubscribers[name] != null){
        console.log('Socket');
        this.socketWebSubscribers[name] = null;
        delete this.socketWebSubscribers[name];
    }
    
    if (this.pmSubscribers[name] != null){
        console.log('PM');
        this.pmSubscribers[name] = null;
        delete this.pmSubscribers[name];
    }
}

AppspaceGeneralServices.prototype.onRemoteSocketReceivedHandler = function(calle,data){
    calle.onRemoteSocketReceived(data);
}

AppspaceGeneralServices.prototype.onRemoteSocketReceived = function(data){
    console.log('sending remote socket data');

    if (data.toLowerCase().indexOf('?actionscript=') == -1 && data.toLowerCase().indexOf('?actionevent=') == -1 && data.toLowerCase().indexOf('?actionremote=') == -1 && data.toLowerCase().indexOf('?actionrs232=') == -1)
        data = '?actionscript=' + data;

    for (var key in this.socketWebSubscribers){
        var conn = this.socketWebSubscribers[key];
        data = encodeURIComponent(data);
        if (conn != null) {
            conn.send(data);
        }else{
            delete this.socketWebSubscribers[key];
        }
    }
}

AppspaceGeneralServices.prototype.onIRReceivedHandler = function(calle,data){
    calle.onIRReceived(data);
}

AppspaceGeneralServices.prototype.onIRReceived = function(data){
    console.log('sending ir data');

    if (data.toLowerCase().indexOf('?actionrawremotecode=') == -1)
        data = '?actionrawremotecode=' + data;

    for (var key in this.irWebSubscribers){
        var conn = this.irWebSubscribers[key];
        if (conn != null) {
            conn.send(data);
        }else{
            delete this.irWebSubscribers[key];
        }
    }
}

AppspaceGeneralServices.prototype.onPMReceivedHandler = function(calle,data){
    return calle.onPMReceived(data);
}

AppspaceGeneralServices.prototype.onPMReceived = function(data){
    console.log('sending pm data');
    this.pmData = data;
    var send = false;
    for (var key in this.pmSubscribers){
    	//console.log('sending to ' + key);
        var conn = this.pmSubscribers[key];
        if (conn !== null) {
            conn.send(data);
            send = true;
        }else{
            delete this.pmSubscribers[key];
        }
    }
    return send;
}

exports.AppspaceGeneralServices = AppspaceGeneralServices;