var fs = require('fs');
var exec = require('child_process').exec;
var ws = require('ws');
var EventHandler = require('../../util/eventhandler.js');
var IRType = {
	EV_KEY : 1,
	EV_MSC : 4
};

var IRState = {
	Released : 0,
	Pressed : 1,
	Repeat : 2
};

function IRReader(){
	this.readStream = null;
	this.irDevice = null;
	
	this.irCode = "";
    this.remoteServer = null;
    this.connectedWebClients = new Object();

    this.lastReceiveIRTime = null;
    this.lastReceiveIRCode = null;
    this.remotePort = 60104;

    this.subscribers = new Object();
}

IRReader.prototype.start = function(){
    var calle = this;
    //start one-way communication web server
    this.remoteServer = new ws.Server({port:this.remotePort,host:'127.0.0.1'});
    this.remoteServer.on('connection',function(connection){
        var connectionName =connection._socket.remoteAddress + ":" + connection._socket.remotePort;
        //console.log(connectionName + ' connected');
        connection.name = connectionName;

        connection.on('close',function(code,reason){
           //console.log(this.name + ' closed');
            calle.connectedWebClients[connection.name] = null;
            delete calle.connectedWebClients[connection.name];
        });

        calle.connectedWebClients[connection.name] = connection;
    });

    //start check ir handler
	var payload = 'ir-keytable';
	exec(payload,EventHandler.CreateEventHandler(this,this.onReceiveFromIRKeytableHandler));

    //start check keyboard handler
    //payload = 'ls -ltra /dev/input/by-id | grep kbd';
    //exec(payload,new EventHandler(this,this.onReceiveFromKeyboardListHandler));
}

IRReader.prototype.stop = function(){
    if (this.remoteServer != null)
        this.remoteServer.close();
}

IRReader.prototype.sendDataToConnectedClients = function(data){
    for (var key in this.connectedWebClients){
        var connection = this.connectedWebClients[key];
        if (connection != null){
            connection.send(data);
        }
    }

    if (this.subscribers != null){
        for (var key in this.subscribers){
            if (this.subscribers[key] != null)
                this.subscribers[key](data);
            else{
                this.subscribers[key] = null;
                delete this.subscribers[key];
            }
        }
    }
}

IRReader.prototype.subscribe = function(func){
    var d = new Date();
    this.subscribers[d.getTime() + '_subscriber'] = func;
}

IRReader.prototype.unsubscribe = function(func){
    for (var key in this.subscribers){
        this.subscribers[key] = null;
        delete this.subscribers[key];
    }
}

IRReader.prototype.onReceiveFromIRKeytableHandler = function(calle,error,stdout,stderr){
	calle.onReceiveFromIRKeytable(error,stdout,stderr);
}

IRReader.prototype.onReceiveFromIRKeytable = function(error,stdout,stderr){
	if (stderr != "" && stderr != null){//ir keytable returns output to stderr
		var str = stderr.toString();
		var start = str.indexOf("(/dev");
		if (start != -1){
			var end = str.indexOf(")",start);
			var irdevice = str.substring(start + 1,end);
			if (irdevice != "" && irdevice != null)
				this.startIRStream(irdevice)
		}
	}
}

IRReader.prototype.startIRStream = function(irDevice){
	this.irDevice = irDevice;
	var response = null;
	this.readStream = fs.createReadStream(this.irDevice,{ 'bufferSize':4*1024});
	this.readStream.on('readable',EventHandler.CreateEventHandler(this,this.onIRReadHandler));
	this.readStream.on('end',function(){/*console.log('ended');*/});
}

IRReader.prototype.onIRReadHandler = function(calle){
	calle.onIRRead();
};

IRReader.prototype.onIRRead = function(){
	var buf;
	while(buf = this.readStream.read()){
		//console.log('Raw:',buf);
		var time = buf.readUInt32LE(0);
		var type = buf.readUInt16LE(8);
		var code = buf.readUInt16LE(10);
		var value = buf.readInt32LE(12).toString(16);

        if (type == IRType.EV_MSC) {
            //console.log(time + ':' + type + ':' + code + ':' + value);

            var irCode = value;
            var state = IRState.Pressed;
            var c = new Date();
            if (this.lastReceiveIRTime == null)
                this.lastReceiveIRTime = new Date(c.getTime());

            if (c.getTime() - this.lastReceiveIRTime.getTime() < 50)
                return;

            if (irCode != this.lastReceiveIRCode)
                state = IRState.Pressed;
            else if (irCode == this.lastReceiveIRCode && (c.getTime() - this.lastReceiveIRTime.getTime() < 150))
                state = IRState.Repeat;

            //console.log(state + ':' + irCode + ':' + (c.getTime() - this.lastReceiveIRTime.getTime()));

            this.lastReceiveIRCode = irCode;
            this.lastReceiveIRTime = new Date(c.getTime());
            this.sendKeyEvent({state:state,code:irCode});
        }
	}
};

IRReader.prototype.sendKeyEvent = function(state){
	var stateString = "";
	switch(state.state){
		case IRState.Released:
			stateString = "released";
			break;
		case IRState.Pressed:
			stateString = "pressed";
			break;
		case IRState.Repeat:
			stateString = "repeat";
			break;
	}
	//console.log("state:" + stateString + " scanCode:" + state.code);
    this.sendDataToConnectedClients(stateString + ',' + state.code);
}

IRReader.prototype.onReceiveFromKeyboardListHandler = function(calle,error,stdout,stderr){
    calle.onReceiveFromKeyboardList(error,stdout,stderr);
}

IRReader.prototype.onReceiveFromKeyboardList = function(error,stdout,stderr){
    if (stdout != "" && stdout != null){
        var str = stdout.toString();
        //console.log(str);
    }
}

exports.IRPRoxyService = IRReader;