var DeviceType = require('../../util/devicetype.js');
var EventHandler = require('../../util/eventhandler.js');
var RequestUtil = require('../../util/requestUtil');
var fs = require('fs');
var exec = require('child_process').exec;
var CommService = require('./commservice').CommunicationService;
var PMUtil = require('./pmutil.js');
var AppspacePlayerProperties = require('../../util/appspaceplayerproperties');
function AppspacePMService() {
	this.deviceType = null;
	this.basePath = null;
	this.nodeid = null;
	this.appspaceip = null;
	this.onPMWrittenHandler = null;
	this.onFlagWrittenHandler = null;
	this.pmfile = null;
	this.startupflag = null;
	this.ispmrunning = false;
	this.commService = null;
	this.pmObject = null;
	this.checkTimer = null;

	this.checkPMValidityEventHandler = null;
    
    var onReadyToStartEventHandler = null;
    var subscribers = {};
    var nodefile = '';
	var deviceDMPusername = null;
	var deviceDMPpassword = null;
	
	var previousMedia = null;
	var isRunningSignApp = true;
	var previousIsRunningSignApp = true;
	
	var doCheckCustomStartupEventHandler = null;
	
	this.setConfig = function(config) {
		this.deviceType = config.deviceType;
		if (config.basePath !== null) {
			this.basePath = config.basePath;
		}
	};

	this.start = function() {
		console.log('starting priority messaging');
		switch (this.deviceType) {
		case DeviceType.Edge300:
		case DeviceType.Edge340:
			this.basePath = '/apps/nexus2';
			break;
		case DeviceType.DMP4310:
		case DeviceType.DMP4400:
			this.basePath = '/tmp/ftproot/usb_1';
			break;
		}
		this.pmfile = this.basePath + '/pmstartupurl';
		this.startupflag = this.basePath + '/startupURLflags';
		nodefile = this.basePath + '/node.dat';

        onReadyToStartEventHandler = EventHandler.CreateEventHandler(this,this.onReadyToStartHandler);
        doCheckCustomStartupEventHandler = EventHandler.CreateEventHandler(this,this.checkCustomStartupHandler);
        
		if (this.deviceType === DeviceType.DMP4310){
			fs.readFile(this.basePath + '/debugScript.txt',EventHandler.CreateEventHandler(this,this.onDebugScriptReadHandler));
		}else{
			PMUtil.isReadyToStart(this.deviceType,onReadyToStartEventHandler);
		}
		
        
		//fs.readFile(nodefile, EventHandler.CreateEventHandler(this, this.onFileConfigReadHandler));
	};
	
	this.onDebugScriptReadHandler = function(calle,err,data){
		calle.onDebugScriptRead(err,data);
	};
	
	this.onDebugScriptRead = function(err,data){
		if (err === null){//file read
			data = data.toString();
			//get dmp username
			deviceDMPusername = AppspacePlayerProperties.retrievePropertiesFromScriptData(data,'Cisco.DMP.DeviceUsername');
			//get dmp password
			deviceDMPpassword = AppspacePlayerProperties.retrievePropertiesFromScriptData(data,'Cisco.DMP.DevicePassword');
			//console.log('dmp username:' + deviceDMPusername);
			//console.log('dmp password:' + deviceDMPpassword);
		}else{
			console.log('error reading debugScript.txt file');
		}
		
		PMUtil.isReadyToStart(this.deviceType,onReadyToStartEventHandler);
	};

    this.subscribe = function(func){
		console.log('adding subscriber on pm');
        var d = new Date();
        subscribers[d.getTime() + '_subscriber'] = func;
    };
    
    this.onReadyToStartHandler = function(calle){
		console.log('ready to start pm');
		calle.checkCustomStartup();
		calle.loadPlayerConfigurationFile();
    };
    
    this.startCheckCustomStartup = function(){
    	setTimeout(doCheckCustomStartupEventHandler,45000);
    };
    
    this.checkCustomStartupHandler = function(calle){
    	calle.checkCustomStartup();
    };
    
    this.checkCustomStartup = function(){
    	var file = this.basePath + '/startupurl';
    	console.log('checking custom startup file:' + file);
    	fs.readFile(file,EventHandler.CreateEventHandler(this,this.onCustomStartupFileReadHandler));
    };
    
    this.onCustomStartupFileReadHandler = function(calle,err,data){
    	calle.onCustomStartupFileRead(err,data);
    };
    
    this.onCustomStartupFileRead = function(err,data){
    	//if file exists, not running sign app
    	isRunningSignApp = (err === null)?false:true;
    	console.log('is running sign manager app?' + isRunningSignApp);
    	this.startCheckCustomStartup();
    };
	
	this.loadPlayerConfigurationFile = function(){
		console.log('loading player configuration file:' + nodefile);
		//read node file
        fs.readFile(nodefile, EventHandler.CreateEventHandler(this, this.onFileConfigReadHandler));
	};

	this.onFileConfigReadHandler = function(calle, err, data) {
		if (err === null) {
			data = data.toString();
			var splitter = (data.indexOf('\r\n') === -1) ? '\n' : '\r\n';
			var lines = data.split(splitter);
			calle.onFileConfigRead(lines);
		} else {
			console.log('Error during loading file config');
			console.log(err);
		}
	};

	this.onFileConfigRead = function(lines) {
		if (lines.length > 0) {
			this.nodeid = lines[0];
			this.appspaceip = lines[2];
		}

		if (this.nodeid !== null && this.appspaceip !== null) {
			console.log('Appspace IP:' + this.appspaceip);
			console.log('Node ID:' + this.nodeid);
			this.startPM();
		} else {
			console.log('Node ID or Appspace IP is null');
			console.log('Node ID:' + this.nodeid);
			console.log('Appspace IP:' + this.appspaceip);
		}
	};

	this.startPM = function() {
		console.log('start priority messaging');
		this.commService = new CommService({
			appspaceURL : this.appspaceip,
			playerID : this.nodeid
		});
		this.commService.onReceiveUpdates = EventHandler.CreateEventHandler(this, this.onReceiveUpdatesHandler);
		this.commService.onReceiveError = EventHandler.CreateEventHandler(this, this.onReceiveErrorHandler);
		this.onSuccessHandler = EventHandler.CreateEventHandler(this, this.onPMReceivedHandler);
		this.onFailHandler = EventHandler.CreateEventHandler(this, this.onPMFailedHandler);
		this.onXMLParsedHandler = EventHandler.CreateEventHandler(this, this.onPMResponseXMLParsedHandler);
		this.onPMWrittenHandler = EventHandler.CreateEventHandler(this, this.onPMFileWrittenHandler);
		this.onFlagWrittenHandler = EventHandler.CreateEventHandler(this, this.onFlagFileWrittenHandler);
		this.checkPMValidityEventHandler = EventHandler.CreateEventHandler(this, this.onDoCheckValidityHandler);
		
		
		
		this.deleteObsoleteFiles();
		this.startCheckPMValidity();
		this.commService.start();
	};

	this.onReceiveUpdatesHandler = function(calle, data) {
		calle.onReceiveUpdates(data);
	};

	this.onReceiveUpdates = function(data) {
		this.pmObject = data;
	};

	this.onReceiveErrorHandler = function(calle, error) {
		calle.onReceiveError(error);
	};

	this.onReceiveError = function(error) {
		console.log('Receive Error!');
		console.log(error);
	};

	this.startCheckPMValidityHandler = function(calle) {
		calle.startCheckPMValidity();
	};

	this.startCheckPMValidity = function() {
		setTimeout(this.checkPMValidityEventHandler, 5000);
	};

	this.onDoCheckValidityHandler = function(calle) {
		calle.onDoCheckValidity();
	};

	this.onDoCheckValidity = function() {
		//console.log('---------');
		//console.log('checking pm validity');
		//console.log('is pm running:' + this.ispmrunning);
		var d = new Date();
		var restartTimer = false;

		if (this.pmObject !== null) {
			var valid = this.pmObject.isValid(d);
			//console.log('is current object valid?' + valid);
			//console.log('same media?' + (this.pmObject.Media === previousMedia));			
			// pm is currently running but the pm object is not valid anymore.
			// so we need to remove the pm
			if (this.ispmrunning === true && valid === false) {
				console.log('stopping pm');
				this.executePMisNotRunning();
				this.ispmrunning = false;
				restartTimer = true;
				
			}
			// pm is currently not running but the pm object is valid
			// pm is running but media changes
			// run pm
			else if (
				(this.ispmrunning === false && valid === true) ||
				(this.ispmrunning === true && valid === true && previousMedia !== this.pmObject.Media)
				){
				console.log('running pm');
				previousMedia = this.pmObject.Media;
				this.executePMisRunning();
				this.ispmrunning = true;
				previousIsRunningSignApp = isRunningSignApp;
				restartTimer = true;
			}
			//if flag changes while pm is running
			else if (this.ispmrunning === true && previousIsRunningSignApp !== isRunningSignApp){
				console.log('flag changes while pm is running, re run pm');
				previousMedia = this.pmObject.Media;
				this.executePMisRunning();
				previousIsRunningSignApp = isRunningSignApp;
				restartTimer = true;
			}
			//if the current status is the same as previous status, don't do anything
			else if (this.ispmrunning === valid){
				console.log('idle');
				restartTimer = true;
			}
		} else {
			restartTimer = true;
			console.log('no PM object');
		}
		//console.log('restart timer?' + restartTimer);
		
		previousIsRunningSignApp = isRunningSignApp;
		
		if (restartTimer === true){
			this.startCheckPMValidity();
		}
	};
	
	this.executePMisRunning = function(){
		var body = "{'module':'pm','enable':'true','media':'"+this.pmObject.retrieveMediaURL(this.appspaceip)+"','mediaType':'"+this.pmObject.MediaType+"'}";
	    var sent = this.sendResponseToSubscribers(body);
		this.send4310Message(body);
		//console.log('sent to subscribers?' + sent);
		//if not running sign manager app and didn't send any response to subscribers
		if (isRunningSignApp === false && sent === false){
			this.savePMFile(this.pmObject.generateURL(this.appspaceip,this.deviceType));
		}
	};
	
	this.executePMisNotRunning = function(){
		var body =  "{'module':'pm','enable':'false','media':'','mediaType':''}";
	    var sent = this.sendResponseToSubscribers(body);
		this.send4310Message(body);
		//console.log('sent to subscribers?' + sent);
		//if not running sign manager app and didn't send any response to subscribers
		if (isRunningSignApp === false && sent === false){
			this.deletePMFile();
		}
	};
	
	this.send4310Message = function(message){
		if (this.deviceType === DeviceType.DMP4310){
			var options = RequestUtil.createOptionsObject();
			options.url = 'https://127.0.0.1/send_message';
			options.method = 'POST';
			options.body = "name=networkmessage&payload=" + message;
			options.authentication.username = deviceDMPusername;
			options.authentication.password = deviceDMPpassword;
			options.failCallback = function(err){console.log(err);};
			options.successCallback = function(res,body){console.log(body);};
			RequestUtil.makeRequest(options);
		}
	};
	
	this.sendResponseToSubscribers = function(data){
		var send = false;
	    if (subscribers !== null){
            for (var key in subscribers){
                if (subscribers[key] !== null){
                    send = subscribers[key](data);
                }else{
                    subscribers[key] = null;
                    delete subscribers[key];
                }
            }
        }
	    return send;
	};

	this.savePMFile = function(url) {
		console.log('writing (' + url + ') file:' + this.pmfile);
		fs.writeFile(this.pmfile, url, this.onPMWrittenHandler);
	};

	this.deletePMFile = function() {
		console.log('writing startupflags');
		fs.writeFile(this.startupflag, "SPLASH=false", this.onFlagWrittenHandler);
	};

	this.deleteObsoleteFiles = function(){
		fs.unlink(this.startupflag,function(err){});
		fs.unlink(this.pmfile, function(err){});
	};
	
	this.onFlagFileWrittenHandler = function(calle, err) {
		calle.onFlagFileWritten(err);
	};

	this.onFlagFileWritten = function(err) {
		if (err !== null) {
			console.log('error during writing/deleting startup URL flag');
			console.log(err.message);
		}

		console.log('unlinking file:' + this.pmfile);
		fs.unlink(this.pmfile, this.onPMWrittenHandler);
	};

	this.onPMFileWrittenHandler = function(calle, err) {
		calle.onPMFileWritten(err);
	};

	this.onPMFileWritten = function(err) {
		if (err !== null) {
			console.log('error during writing/deleting pm file');
			console.log(err.message);
		}

		console.log('killing chrome');
		exec('killall -9 chrome');

		this.onPMRequested();
	};

	this.onPMFailedHandler = function(calle, err) {
		calle.onPMFailed(err);
	};

	this.onPMFailed = function(err) {
		console.log('error during request');
		console.log(err.message);
		// this.onPMRequested();
		//this.deletePMFile();
	};

	this.onPMRequested = function() {
		//this.startCheckPMValidity();
	};
}

exports.AppspacePMService = AppspacePMService;