function PriorityMessagingObject() {
	this.StartDate = null;
	this.EndDate = null;
	this.Media = '';
	this.MediaType = '';
	this.Recurrence = 'none';
	this.StateGuid = '';
	
	this.isValid = function(date) {
		var res = false;
		if (date === null) {
			date = new Date();
		}

		switch (this.Recurrence) {
		case 'none':
			res = this.isCurrentValid(date);
			break;
		case 'daily':
			res = this.isDailyValid(date);
			break;
		case 'weekly':
			res = this.isWeeklyValid(date);
			break;
		case 'monthly':
			res = this.isMonthlyValid(date);
			break;
		}

		return res;

	};

	this.isCurrentValid = function(current) {
		return (current.getTime() >= this.StartDate.getTime() && current.getTime() <= this.EndDate.getTime());
	};

	this.isDailyValid = function(current) {
		var startdate = new Date(this.StartDate.getTime());
		startdate.setDate(current.getDate());
		startdate.setMonth(current.getMonth());
		startdate.setYear(current.getFullYear());

		var diff = startdate.getTime() - this.StartDate.getTime();
		var enddate = new Date(this.EndDate.getTime() + diff);
		return (current.getTime() >= startdate.getTime() && current.getTime() <= enddate.getTime());
	};

	this.isWeeklyValid = function(current) {
		var res = false;
		// get current day
		var targetDay = this.StartDate.getDay();
		// only check if the current day is the same as targeted day
		if (targetDay === current.getDay()) {
			res = this.isDailyValid(current);
		}
		return res;
	};

	this.isMonthlyValid = function(current) {
		var res = false;
		var targetDate = this.StartDate.getDate();
		// get total days in current month
		var maxDayInAMonth = new Date(current.getFullYear(), current.getMonth() + 1, 0);

		// if the targeted date is more than the total days of current month,
		// use the max number of days of the month
		if (targetDate > maxDayInAMonth) {
			targetDate = maxDayInAMonth;
		}

		// only check if the current date is the same targeted date
		if (targetDate === current.getDate()) {
			res = this.isDailyValid(current);
		}
		return res;
	};

	this.generateURL = function(appspaceip,deviceType){
		var base = 'file:///apps/nexus2/pages/pm/';
		var res = base + 'prioritymessaging.html?media=' + this.Media.split('{APPSPACEIP}').join(appspaceip).split(' ').join('%20') + '&type=' + this.MediaType;
		return res;
	};
	
	this.retrieveMediaURL = function(appspaceip){
		return this.Media.split('{APPSPACEIP}').join(appspaceip).split(' ').join('%20');
	};
	
	this.copy = function(object) {
		if (object.StartedAt !== undefined) {
			this.StartDate = new Date(object.StartedAt);
		}

		if (object.EndedAt !== undefined) {
			this.EndDate = new Date(object.EndedAt);
		}

		if (object.Media !== undefined) {
			this.Media = object.Media;
			this.Media = this.Media.split('\\').join('/');
			this.Media = this.Media.toLowerCase().split('c:/fastnetv2/ug media/').join('{APPSPACEIP}/app/fastnetdata/ug media/');
			this.Media = this.Media.split(' ').join('%20');
		}

		if (object.ContentType !== undefined) {
			this.MediaType = object.ContentType;
		}

		if (object.Recurrence !== undefined) {
			switch (object.Recurrence.toLowerCase()) {
			case '0':
			case 'none':
				this.Recurrence = 'none';
				break;
			default:
				this.Recurrence = object.Recurrence.toLowerCase();
			}
		}

		if (object.TotalSeconds !== undefined) {
			var duration = parseInt(object.TotalSeconds);
			if (duration > 0) {
				this.EndDate = new Date(this.StartDate.getTime());
				this.EndDate.setSeconds(this.EndDate.getSeconds() + duration);
			}
		}
		
		if (object.StateGuid !== undefined){
			this.StateGuid = object.StateGuid;
		}
	};
}

exports.PriorityMessagingObject = PriorityMessagingObject;