#!/bin/bash
REPLY=""
PATH="/usr/java/jre1.7.0_09//bin:/usr/local/cisco/bin:/usr/local/cisco/sbin:/usr/local/bin:/usr/bin:/bin:/usr/local/sbin:/usr/sbin:/sbin:/scripts:/opt/:/opt/chromium/:"
_tempPath="/tmp/appspace"

function Log(){
	local logfile="$_tempPath/appspacemonitor.log"
	
	if [ -f $logfile ] ; then
		OLDIFS=$IFS
		line=`stat -t $logfile`
		unset TEMPARR
		IFS=' ' read -ra TEMPARR <<< "$line"
		local logSize="${TEMPARR[1]}"
		IFS=$OLDIFS	
		if [ ${logSize} -gt 30240000 ] ; then
			rm -rf "$logfile"
		fi
	else
		touch $logfile
	fi
	
	local d=$(date +%m/%d/%y-%T)
	echo "$d: ${1}" >> "$logfile"
	echo "$d: ${1}"
}

function SaveLog(){
	cp -f $_tempPath/appspacemonitor.log /apps/nexus2/Logs/appspacemonitor.log
}

function ProcessData(){
	local data="$1"
	local tmp="$_tempPath/monitor.process.data"
	echo "$data" > "$tmp"
	dos2unix $tmp > /dev/null
	_REPLY=`cat $tmp`
	rm -f $tmp
}

function CheckDownloadManager(){
	local file="$_tempPath/dmheartbeat"
	Log "START CheckDownloadManager"
	
	local secondstorestart=600
	if [ -f /apps/nexus2/debugScript.txt ] ; then
		local check=`cat /apps/nexus2/debugScript.txt | grep Download.Manager.Heartbeat.Check.Interval=`
		if [ "$check" != "" ] ; then
			check=`echo "$check" | sed 's/Download.Manager.Heartbeat.Check.Interval=//g'`
			check=`echo "$check" | sed 's/;//g'`
			ProcessData "$check"
			check="$_REPLY"
			if [ `echo "$check" | grep "^[0-9]*$"` ]; then
				secondstorestart=$(($check*1))
			fi
		fi
	fi
	
	local secondstoreboot=$(($secondstorestart * 2))
	Log "Config: Seconds to restart download manager : $secondstorestart"
	Log "Config: Seconds to reboot : $secondstoreboot"
	
	local storage="/apps/nexus2"
	
	local currentTime=`date +%s`
	Log "Current time : $currentTime"
	
	Log "Check Temp Downloads folder : $storage/Temp Downloads"
	local continue="true"
	local doreboot="false"
	local isfiledownloading="false"
	local downloadingfile=""
	local OLDIFS=$IFS
	IFS=$'\n'
	for f in $(find "$storage/Temp Downloads" -type f -print)
	do
		currentTempFileTime=$(date -r $f +%s)
		Log "Last modified $f : $currentTempFileTime"
		local subdiff=$(($currentTime-$currentTempFileTime))
		if [ $subdiff -gt 60 ] ; then
			Log "$f is modifed $subdiff ago. Possible hang. Killing downloadmanager"
			continue="false"
			isfiledownloading="false"
			break
		else
			isfiledownloading="true"
			downloadingfile="$f"
		fi
	done
	IFS=$OLDIFS
	
	if [ "$isfiledownloading" = "true" ] ; then
		Log "Download manager is downloading file : $downloadingfile. Skip heartbeat check"
		return
	elif [ "$continue" = "true" ] ; then
		Log "Checking Download Manager heartbeat : $file"
		if [ -f $file ] ; then
			:
		else
			Log "File does not exist : $file"
			return
		fi
		local currentFileTime=`date -r $file +%s`
		Log "Current heartbeat file time : $currentFileTime"
		
		local diff=$(($currentTime-$currentFileTime))
		Log "Heartbeat difference: $diff"
		
		if [ $diff -gt $secondstoreboot ] ; then
			doreboot="true"
			continue="false"
			Log "Heartbeat is not updated for more than $secondstoreboot seconds, rebooting device"
		elif [ $diff -gt $secondstorestart ] ; then
			continue="false"
			Log "Heartbeat is not updated for more than $secondstorestart seconds, killing downloadmanager"
		fi
	fi
	
	if [ "$doreboot" = "true" ] ; then
		SaveLog
		/bin/bash /apps/nexus2/rebootscript.sh
	elif [ "$continue" = "false" ] ; then
		Log "Killing DM using pkill"
		pkill -9 -f downloadmanager
	fi
	
	Log "END CheckDownloadManager"
}

function CheckAppspaceChromeLog(){
	Log "START CheckAppspaceChromeLog"
	if [ -f $_tempPath/appspacechrome.log ] ; then
		local tmpFileTime=`date -r $_tempPath/appspacechrome.log +%s`
		if [ -f /apps/nexus2/Logs/appspacechrome.log ] ; then
			local targetFileTime=`date -r /apps/nexus2/Logs/appspacechrome.log +%s`
			if [[ $tmpFileTime ==  $targetFileTime ]] ; then
				Log "Chrome log files timestamp are the same (source:$tmpFileTime, target:$targetFileTime)."
			else
				Log "Chrome log files timestamp are different (source:$tmpFileTime, target:$targetFileTime). Copying to /apps/nexus2/Logs"
				cp -f -p $_tempPath/appspacechrome.log /apps/nexus2/Logs/appspacechrome.log
			fi
		else
			Log "Chrome log not exists. Copying to /apps/nexus2/Logs"
			cp -f -p $_tempPath/appspacechrome.log /apps/nexus2/Logs/appspacechrome.log
		fi
	fi
	Log "END CheckAppspaceChromeLog"
}

function Start(){
	
	[[ -d $_tempPath ]] && : || mkdir -p $_tempPath
	
	Log "START Appspace Monitor"
	CheckDownloadManager
	CheckAppspaceChromeLog	
	Log "END Appspace Monitor"
}

Start
