#!/bin/bash

_res=""

function getDevice(){
	local ts340=`cat /etc/*-release | grep VERSION=`
	local ts300=""
	if [ -f /scripts/cisco_env.sh ] ; then
		ts300=`cat /scripts/cisco_env.sh | grep FS_VERSION=`
	fi
	_res=""	
	if [ "$ts340" != "" ] ; then
		_res="e340"
	elif [ "$ts300" != "" ] ; then
		_res="e300"
	fi
}

function startInstallation(){
	echo ""
	echo "Start Installation"
	local installationScript="installation"
	getDevice
	if [ "$_res" = "e300" ] ; then
		installationScript="$installationScript""_300.sh"
	elif [ "$_res" = "e340" ] ; then
		installationScript="$installationScript""_340.sh"
	else
		installationScript=""
	fi
	
	if [ "$installationScript" = "" ] ; then
		echo "No device type found (Receive $_res). Exiting"
		echo "Error;No device type found (Receive $_res)" > "/apps/nexus2/finaloutput.txt"
		exit
	fi
	
	if [ -f "$installationScript" ] ; then
		:
	else
		echo "No installation script found. Looking for $installationScript. Exiting"
		echo "Error;No installation script found. Looking for $installationScript" > "/apps/nexus2/finaloutput.txt"
		exit
	fi

	local appspaceip=""
	local apikey=""
	local tag=""
	local devicename=""
	local deviceip=""
	local networkid=""
	
	for i in "$@"
	do
		if [[ "$i" = "appspaceip"* ]] ; then
			appspaceip=`echo "$i" | sed 's/appspaceip=//g'`
		elif [[ "$i" = "apikey"* ]] ; then
			apikey=`echo "$i" | sed 's/apikey=//g'`
		elif [[ "$i" = "tag"* ]] ; then
			tag=`echo "$i" | sed 's/tag=//g'`
		elif [[ "$i" = "devicename"* ]] ; then
			devicename=`echo "$i" | sed 's/devicename=//g'`
		elif [[ "$i" = "deviceip"* ]] ; then
			deviceip=`echo "$i" | sed 's/deviceip=//g'`
		elif [[ "$i" = "networkid"* ]] ; then
			networkid=`echo "$i" | sed 's/networkid=//g'`
		fi
	done
	echo ""
	echo "Received parameters:"
	echo "appspaceip=$appspaceip"
	echo "apikey=$apikey"
	echo "tag=$tag"
	echo "devicename=$devicename"
	echo "deviceip=$deviceip"
	echo "networkid=$networkid"
	echo ""
	echo "Stopping Processes"
	./"$installationScript" "stopprocesses"
	echo ""
	echo "Configure Device"
	./"$installationScript" "configuredevice"
	echo ""
	echo "Binding Device to Appspace"
	./"$installationScript" "binddevice" "appspaceip=$appspaceip" "apikey=$apikey" "tag=$tag" "devicename=$devicename" "deviceip=$deviceip" "networkid=$networkid"
}

startInstallation "$@"