#!/bin/bash
PATH="/sbin:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/bin:/root/bin"
_res=""
_finalOutputFile="/apps/nexus2/finaloutput.txt"

function StopProcess()
{
	local temp=`ps aux | grep "$1" | grep -v grep`
	OLDIFS=$IFS
	unset TEMPARR
	IFS=' ' read -ra TEMPARR <<< "$temp"
	_res=""
	_res="${TEMPARR[1]}"
	if [ "$_res" != "" ] ; then
			kill "$2" "$_res"
	fi
	IFS=$OLDIFS
}

function BindDevice()
{
	local appspaceip=""
	local apikey=""
	local tag=""
	local devicename=""
	local deviceip=""
	local networkid=""
	
	for i in "$@"
	do
		if [[ "$i" = "appspaceip"* ]] ; then
			appspaceip=`echo "$i" | sed 's/appspaceip=//g'`
		elif [[ "$i" = "apikey"* ]] ; then
			apikey=`echo "$i" | sed 's/apikey=//g'`
		elif [[ "$i" = "tag"* ]] ; then
			tag=`echo "$i" | sed 's/tag=//g'`
		elif [[ "$i" = "devicename"* ]] ; then
			devicename=`echo "$i" | sed 's/devicename=//g'`
		elif [[ "$i" = "deviceip"* ]] ; then
			deviceip=`echo "$i" | sed 's/deviceip=//g'`
		elif [[ "$i" = "networkid"* ]] ; then
			networkid=`echo "$i" | sed 's/networkid=//g'`
		fi
	done

	#get mac
	local ts=`ifconfig | grep HWaddr`
	ts=`echo "$ts" | sed 's/\n//g'`
	OLDIFS=$IFS
	unset TEMPARR
	IFS=' ' read -ra TEMPARR <<< "$ts"
	local mac="${TEMPARR[$((${#TEMPARR[@]}-1))]}"
	IFS=$OLDIFS
	
	#get device guid from Appspace
	if [ "$appspaceip" = "" ] ; then
		echo "Error;No Appspace URL provided" > $_finalOutputFile
	elif [ "$apikey" = "" ] ; then
		echo "Error;No API Key provided" > $_finalOutputFile
	elif [ "$deviceip" = "" ] ; then
		echo "Error;No device IP provided" > $_finalOutputFile
	else
		local getguidUrl="$appspaceip/integrationwebservices/LicenseHandler.ashx?query=license.getdeviceguid&api=$apikey&di=$mac"
		local deviceguid=$(curl -k -s --connect-timeout 30 --url "$getguidUrl")
		echo ""
		echo "Retrieve device GUID"
		echo "Requesting $getguidUrl"
		echo "Receive $deviceguid"
		if [ "$deviceguid" != "" ] ; then
			#check if device bounded
			echo ""
			echo "Check if device is bounded"
			local isdeviceboundedUrl="$appspaceip/integrationwebservices/LicenseHandler.ashx?query=license.isdevicebounded&api=$apikey&dguid=$deviceguid"
			local isdevicebounded=$(curl -k -s --connect-timeout 30 --url "$isdeviceboundedUrl")
			echo "Requesting $isdeviceboundedUrl"
			echo "Receive $isdevicebounded"
			if [ "$isdevicebounded" = "true" ] ; then
				#unbind device
				echo ""
				echo "Device is bounded. Unbind device"
				local unbinddeviceUrl="$appspaceip/integrationwebservices/LicenseHandler.ashx?query=license.unbinddevice&api=$apikey&dguid=$deviceguid"
				local resunbinddevice=$(curl -k -s --connect-timeout 30 --url "$unbinddeviceUrl")
				echo "Requesting $unbinddeviceUrl"
				echo "Receive $resunbinddevice"
			fi
			
			#bind device
			if [ "$devicename" = "" ] ; then
				devicename="$mac"
			fi
			
			echo ""
			echo "Binding device"
			local binddeviceUrl="$appspaceip/integrationwebservices/LicenseHandler.ashx?query=license.binddevice&api=$apikey&dguid=$deviceguid&dname=$devicename&dip=$deviceip"
			
			if [ "$tag" = "" ] ; then
				:
			else
				binddeviceUrl="$binddeviceUrl&tag=$tag"
			fi
			if [ "$networkid" = "" ] ; then
				:
			else
				binddeviceUrl="$binddeviceUrl&netid=$networkid"
			fi
			
			local resbinddevice=$(curl -k -s --connect-timeout 30 --url "$binddeviceUrl")
			echo "Requesting $binddeviceUrl"
			echo "Receive $resbinddevice"
			if [[ "$resbinddevice" = *"Error"* ]] ; then
				echo "Error;Unable to bind device to Appspace. Received error from server." > $_finalOutputFile
			elif [ "$resbinddevice" = "" ] ; then
				echo "Error;Unable to bind device to Appspace. Server returns empty. Player max number exceeded" > $_finalOutputFile
			else
				echo ""
				echo "Retrieve device Node ID"
				local getdevicenodeidUrl="$appspaceip/integrationwebservices/LicenseHandler.ashx?query=license.getdevicenodeid&api=$apikey&di=$mac&dei=$resbinddevice"
				local nodeid=$(curl -k -s --connect-timeout 30 --url "$getdevicenodeidUrl")
				echo "Requesting $getdevicenodeidUrl"
				echo "Receive $nodeid"
				if [ "$nodeid" != "" ] ; then
					echo ""
					echo "Sending model name to Appspace"
					local setpropertyUrl="$appspaceip/integrationwebservices/ConfigScriptHandler.ashx?operation=setproperty&key=Player.Model.Name&value=EDGE340&type=player&id=$nodeid"
					local setpropertyres=$(curl -k -s --connect-timeout 30 --url "$setpropertyUrl")
					echo "Requesting $setpropertyUrl"
					echo "Receive $setpropertyres"
					
					echo "Dumping registration info to /apps/nexus2/node.dat"
					[[ -f /apps/nexus2/node.dat ]] && rm -rf /apps/nexus2/node.dat && touch /apps/nexus2/node.dat
					echo -e "$nodeid\nedge340\n$appspaceip\ndownloadmanager=true" >> /apps/nexus2/node.dat
					echo "Success" > $_finalOutputFile
					echo ""
					echo "Registration successful"
				else
					echo "Error;Receive empty node ID from Appspace" > $_finalOutputFile
				fi
			fi
		else
			echo "Error;Unable to retrieve device GUID" > $_finalOutputFile
		fi
	fi
}

function DumpVersion()
{
	if [ -f /apps/nexus2/internal-version.dat ] ; then
		local version=`cat /apps/nexus2/internal-version.dat`
		[[ ! "$version" = "" ]] && echo "Edge Player=$version" > /apps/nexus2/version.inf
	elif [ -f /apps/nexus2/downloadmanageredge300.sh ] ; then
		local version=`cat /apps/nexus2/downloadmanageredge300.sh | grep "dmversion=" | sed 's/dmversion=//g' | sed 's/"//g'`
		[[ ! "$version" = "" ]] && echo "Edge Player=$version" > /apps/nexus2/version.inf
	fi
}

function Main()
{
	if [ -f /home/appspace/install-reboot ] ; then
		rm -rf /home/appspace/install-reboot
	fi
	
	local res=""
	if [ "$1" = "getworkingfolder" ] ; then
		res="/apps/nexus2"
	elif [ "$1" = "stopprocesses" ] ; then
		StopProcess "chrome" "-2"
		StopProcess "admholder" "-9"
		StopProcess "appspacenode" "-9"
		StopProcess "commholder" "-9"
		StopProcess "appspacecomm" "-9"
		StopProcess "dmholder" "-9"
		StopProcess "downloadmanager" "-9"
		StopProcess "xinetd" "-9"
	elif [ "$1" = "configuredevice" ] ; then
		DumpVersion
	elif [ "$1" = "binddevice" ] ; then		
		BindDevice "$@"
	fi

	echo "$res"
}

Main "$@"