#!/bin/bash

_res=""

function getDevice(){
	local ts340=`cat /etc/*-release | grep VERSION=`
	local ts300=""
	if [ -f /scripts/cisco_env.sh ] ; then
		ts300=`cat /scripts/cisco_env.sh | grep FS_VERSION=`
	fi
	_res=""	
	if [ "$ts340" != "" ] ; then
		_res="e340"
	elif [ "$ts300" != "" ] ; then
		_res="e300"
	fi
}

function Switch()
{
	local operation="$1"
	getDevice
	local device="$_res"
	if [[ "$1" = *"desktop"* ]] ; then
	
		if [ "$device" = "e300" ] ; then
			local res=""
			if [ -f /apps/nexus2/device.config ] ; then
				:
			else
				res=`curl -s -k -X GET -H 'password:cisco' -H 'Content-Type: application/json' https://127.0.0.1/api/1.0/sys/ntpServer`
				local ntp=`/apps/nexus2/node -pe 'JSON.parse(process.argv[1]).ntpServer' "$res"`
				
				res=`curl -s -k -X GET -H 'password:cisco' -H 'Content-Type: application/json' https://127.0.0.1/api/1.0/sys/locale`
				local timezone=`/apps/nexus2/node -pe 'JSON.parse(process.argv[1]).locale' "$res"`
				
				res=`curl -s -k -X GET -H 'password:cisco' -H 'Content-Type: application/json' https://127.0.0.1/api/1.0/sys/resolution`
				local resolution=`/apps/nexus2/node -pe 'JSON.parse(process.argv[1]).resolution' "$res"`
				
				echo "ntpserver=$ntp" > /apps/nexus2/device.config
				echo "timezone=$timezone" >> /apps/nexus2/device.config
				echo "resolution=$resolution" >> /apps/nexus2/device.config
			fi
			echo "Enable login GUI"
			res=`curl -s -k -X PUT -H 'password:cisco' -H 'Content-Type: application/json' -d '{"loginGui" : "enable"}' https://127.0.0.1/api/1.0/sys/loginGui`
		fi
		
		[[ -f /apps/nexus2/desktop-enable ]] && rm -rf /apps/nexus2/desktop-enable
		[[ -f /apps/nexus2/externalfiles/xinitrc.original ]] && cp -f /apps/nexus2/externalfiles/xinitrc.original /etc/xdg/xfce4/xinitrc
		[[ -f /apps/nexus2/externalfiles/xfce4-keyboard-shortcuts.xml.original ]] && cp -f /apps/nexus2/externalfiles/xfce4-keyboard-shortcuts.xml.original /home/user/.config/xfce4/xfconf/xfce-perchannel-xml/xfce4-keyboard-shortcuts.xml
		
		[[ -f /apps/nexus2/externalfiles/services.original ]] && cp -f /apps/nexus2/externalfiles/services.original /etc/services
		[[ -f /apps/nexus2/externalfiles/startxfce4.original ]] && cp -f /apps/nexus2/externalfiles/startxfce4.original /scripts/startxfce4
		[[ -f /apps/nexus2/externalfiles/slim.conf.original ]] && cp -f /apps/nexus2/externalfiles/slim.conf.original /usr/local/cisco/etc/slim.conf
		[[ -f /apps/nexus2/externalfiles/managemode.original ]] && cp -f /apps/nexus2/externalfiles/managemode.original /usr/clish/managemode
		touch /apps/nexus2/desktop-enable
		
	elif [[ "$1" = *"player"* ]] ; then
		[[ -f /apps/nexus2/desktop-enable ]] && rm -rf /apps/nexus2/desktop-enable
		[[ -f /apps/nexus2/externalfiles/xinitrc ]] && cp -f /apps/nexus2/externalfiles/xinitrc /etc/xdg/xfce4/xinitrc
		[[ -f /apps/nexus2/externalfiles/xfce4-keyboard-shortcuts.xml ]] && cp -f /apps/nexus2/externalfiles/xfce4-keyboard-shortcuts.xml /home/user/.config/xfce4/xfconf/xfce-perchannel-xml/xfce4-keyboard-shortcuts.xml
		
		[[ -f /apps/nexus2/externalfiles/services ]] && cp -f /apps/nexus2/externalfiles/services /etc/services
		[[ -f /apps/nexus2/externalfiles/startxfce4 ]] && cp -f /apps/nexus2/externalfiles/startxfce4 /scripts/startxfce4
		[[ -f /apps/nexus2/externalfiles/slim.conf ]] && cp -f /apps/nexus2/externalfiles/slim.conf /usr/local/cisco/etc/slim.conf
		[[ -f /apps/nexus2/externalfiles/managemode ]] && cp -f /apps/nexus2/externalfiles/managemode /usr/clish/managemode
		[[ -f /apps/nexus2/externalfiles/nexuspfconf ]] && cp -f /apps/nexus2/externalfiles/nexuspfconf /etc/xinetd.d/
		[[ -f /apps/nexus2/externalfiles/nexusrtconf ]] && cp -f /apps/nexus2/externalfiles/nexusrtconf /etc/xinetd.d/
		[[ -f /apps/nexus2/externalfiles/nexusstatconf ]] && cp -f /apps/nexus2/externalfiles/nexusstatconf /etc/xinetd.d/
		
		if [ "$device" = "e300" ] ; then
			local res=""
			echo "Disable login GUI"
			res=`curl -s -k -X PUT -H 'password:cisco' -H 'Content-Type: application/json' -d '{"loginGui" : "disable"}' https://127.0.0.1/api/1.0/sys/loginGui`
			if [ -f /apps/nexus2/device.config ] ; then
				local ntpserver=""
				local timezone=""
				local resolution=""
				
				while read config
				do
					if [[ "$config" = *"ntpserver="* ]] ; then
						ntpserver=`echo "$config" | sed 's/ntpserver=//g'`
					elif [[ "$config" = *"timezone="* ]] ; then
						timezone=`echo "$config" | sed 's/timezone=//g'`
					elif [[ "$config" = *"resolution="* ]] ; then
						resolution=`echo "$config" | sed 's/resolution=//g'`
					fi
				done < /apps/nexus2/device.config
				
				if [ "$ntpserver" != "" ] ; then
					echo "Setting up NTP Server ($ntpserver)"
					res=`curl -s -k -X PUT -H 'password:cisco' -H 'Content-Type: application/json' -d '{"ntpServer" : "'"$ntpserver"'"}' https://127.0.0.1/api/1.0/sys/ntpServer`
					echo "Res : $res"
				fi
				if [ "$timezone" != "" ] ; then
					echo "Setting up Locale ($timezone)"
					res=`curl -s -k -X PUT -H 'password:cisco' -H 'Content-Type: application/json' -d '{"locale" : "'"$timezone"'"}' https://127.0.0.1/api/1.0/sys/locale`
					echo "Res : $res"
				fi
				if [ "$resolution" != "" ] ; then
					echo "Setting up Resolution ($resolution)"
					res=`curl -s -k -X PUT -H 'password:cisco' -H 'Content-Type: application/json' -d '{"resolution" : "'"$resolution"'"}' https://127.0.0.1/api/1.0/sys/resolution`
					echo "Res : $res"
				fi
			fi
			echo "Reconfigure Flash player trust"
			[[ -d /etc/adobe/FlashPlayerTrust/ ]] && rm -rf /etc/adobe/FlashPlayerTrust/ && rm -rf /etc/adobe                                   
            mkdir -p /etc/adobe/FlashPlayerTrust/                                                                                               
            echo "/apps/nexus2" > /etc/adobe/FlashPlayerTrust/trust.cfg
			
		fi
	fi
	
	
	[[ -f /etc/xdg/xfce4/xinitrc ]] && dos2unix /etc/xdg/xfce4/xinitrc > /dev/null && chmod 777 /etc/xdg/xfce4/xinitrc
	[[ -f /home/user/.config/xfce4/xfconf/xfce-perchannel-xml/xfce4-keyboard-shortcuts.xml ]] && dos2unix /home/user/.config/xfce4/xfconf/xfce-perchannel-xml/xfce4-keyboard-shortcuts.xml > /dev/null
}

Switch "$1"